/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public class PublicationAuthenticationSession extends SipSession {
  private long swigCPtr;

  protected PublicationAuthenticationSession(long cPtr, boolean cMemoryOwn) {
    super(tinyWRAPJNI.PublicationAuthenticationSession_SWIGUpcast(cPtr), cMemoryOwn);
    swigCPtr = cPtr;
  }

  protected static long getCPtr(PublicationAuthenticationSession obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        tinyWRAPJNI.delete_PublicationAuthenticationSession(swigCPtr);
      }
      swigCPtr = 0;
    }
    super.delete();
  }

  public PublicationAuthenticationSession(SipStack pStack) {
    this(tinyWRAPJNI.new_PublicationAuthenticationSession(SipStack.getCPtr(pStack), pStack), true);
  }

  public boolean publish(String mcptt_info, String poc_settings, java.nio.ByteBuffer payload, long len, ActionConfig config) {
    return tinyWRAPJNI.PublicationAuthenticationSession_publish__SWIG_0(swigCPtr, this, mcptt_info, poc_settings, payload, len, ActionConfig.getCPtr(config), config);
  }

  public boolean publish(String mcptt_info, String poc_settings, java.nio.ByteBuffer payload, long len) {
    return tinyWRAPJNI.PublicationAuthenticationSession_publish__SWIG_1(swigCPtr, this, mcptt_info, poc_settings, payload, len);
  }

  public boolean unPublish(java.nio.ByteBuffer payload, long len, ActionConfig config) {
    return tinyWRAPJNI.PublicationAuthenticationSession_unPublish__SWIG_0(swigCPtr, this, payload, len, ActionConfig.getCPtr(config), config);
  }

  public boolean unPublish(java.nio.ByteBuffer payload, long len) {
    return tinyWRAPJNI.PublicationAuthenticationSession_unPublish__SWIG_1(swigCPtr, this, payload, len);
  }

}
