/*
* Copyright (C) 2010-2011 Mamadou Diop.
*
* Contact: Mamadou Diop <diopmamadou(at)doubango.org>
*	
* This file is part of Open Source Doubango Framework.
*
* DOUBANGO is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*	
* DOUBANGO is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*	
* You should have received a copy of the GNU General Public License
* along with DOUBANGO.
*
*/

/**@file tinydav.h
 * @brief tinyDAV API.
 *
 * @author Mamadou Diop <diopmamadou(at)doubango.org>
 *

 */
#ifndef TINYMEDIA_TINYDAV_H
#define TINYMEDIA_TINYDAV_H

#include "tinydav/tdav.h"

// sessions
#include "tinydav/audio/tdav_session_audio.h"
// codecs
#include "tinydav/codecs/g711/tdav_codec_g711.h"

/* == tinyMEDIA == */
#include "tinymedia.h"

#endif /* TINYMEDIA_TINYDAV_H */
