audio_opensles.o: audio_opensles.cxx audio_opensles.h \
 audio_opensles_config.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/stdint.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/_types.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/machine/_types.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/android/log.h \
 ../../tinySAK/src/tsk_plugin.h ../../tinySAK/src/tinysak_config.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/string.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/cdefs.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/cdefs_elf.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/malloc.h \
 ../../tinySAK/src/tsk_common.h ../../tinySAK/src/tsk_object.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/stdio.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/types.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/posix_types.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/stddef.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/compiler.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/asm/posix_types.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/asm/types.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/types.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/machine/kernel.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/sysmacros.h \
 audio_opensles_consumer.h audio_opensles_producer.h \
 audio_opensles_device.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES_Platform.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES_Android.h \
 C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES_AndroidConfiguration.h \
 audio_opensles_device_impl.h \
 ../../tinyMEDIA/include/tinymedia/tmedia_consumer.h \
 ../../tinyMEDIA/include/tinymedia_config.h \
 ../../tinyMEDIA/include/tinymedia/tmedia_codec.h \
 ../../tinyMEDIA/include/tinymedia/tmedia_common.h \
 ../../tinySAK/src/tsk_object.h ../../tinySAK/src/tsk_list.h \
 ../../tinySAK/src/tsk_mutex.h \
 ../../tinyMEDIA/include/tinymedia/tmedia_params.h \
 ../../tinyMEDIA/include/tinymedia/tmedia_common.h \
 ../../tinyMEDIA/include/tinymedia/tmedia_producer.h \
 ../../tinySAK/src/tsk_safeobj.h ../../tinySAK/src/tsk_debug.h

audio_opensles.h:

audio_opensles_config.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/stdint.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/_types.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/machine/_types.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/android/log.h:

../../tinySAK/src/tsk_plugin.h:

../../tinySAK/src/tinysak_config.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/string.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/cdefs.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/cdefs_elf.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/malloc.h:

../../tinySAK/src/tsk_common.h:

../../tinySAK/src/tsk_object.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/stdio.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/types.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/posix_types.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/stddef.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/compiler.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/asm/posix_types.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/asm/types.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/linux/types.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/machine/kernel.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/sys/sysmacros.h:

audio_opensles_consumer.h:

audio_opensles_producer.h:

audio_opensles_device.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES_Platform.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES_Android.h:

C:/android-ndk-r7c/platforms/android-9/arch-arm/usr/include/SLES/OpenSLES_AndroidConfiguration.h:

audio_opensles_device_impl.h:

../../tinyMEDIA/include/tinymedia/tmedia_consumer.h:

../../tinyMEDIA/include/tinymedia_config.h:

../../tinyMEDIA/include/tinymedia/tmedia_codec.h:

../../tinyMEDIA/include/tinymedia/tmedia_common.h:

../../tinySAK/src/tsk_object.h:

../../tinySAK/src/tsk_list.h:

../../tinySAK/src/tsk_mutex.h:

../../tinyMEDIA/include/tinymedia/tmedia_params.h:

../../tinyMEDIA/include/tinymedia/tmedia_common.h:

../../tinyMEDIA/include/tinymedia/tmedia_producer.h:

../../tinySAK/src/tsk_safeobj.h:

../../tinySAK/src/tsk_debug.h:
