/* 

*  Copyright (C) 2020, University of the Basque Country (UPV/EHU)
*
* Contact for licensing options: <licensing-mcpttclient(at)mcopenplatform(dot)com>
*       
* This file is part of MCOP MCPTT Client
*
* This is free software: you can redistribute it and/or modify it under the terms of
* the GNU General Public License as published by the Free Software Foundation, either version 3 
* of the License, or (at your option) any later version.
*       
* This is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
* See the GNU General Public License for more details.
*       
* You should have received a copy of the GNU General Public License along 
* with this program; if not, write to the Free Software Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
 * @file tmcptt_event.h
 * @brief MCPTT Timers.
 *
 */

#ifndef TINYMCPTT_TIMERS_H
#define TINYMCPTT_TIMERS_H

#include "tinymcptt_config.h"

TMCPTT_BEGIN_DECLS
#define MCPTT_TIMER_TINIT_DEFAULT_VALUE	1000 //Time in ms
#define MCPTT_TIMER_T100_DEFAULT_VALUE	2000 //Time in ms
#define MCPTT_TIMER_T132_MAX_VALUE	6000 //Time in ms
#define MCPTT_TIMER_T101_DEFAULT_VALUE	2000 //Time in ms
#define MCPTT_TIMER_T103_DEFAULT_VALUE	2000 //Time in ms
#define MCPTT_TIMER_T103_DEFAULT_VALUE	2000 //Time in ms
#define MCPTT_TIMER_T132_DEFAULT_VALUE	2000 //Time in ms

TMCPTT_END_DECLS

#endif /* TINYMCPTT_TIMERS_H */