/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public class ProxyAudioProducer extends ProxyPlugin {
  private long swigCPtr;

  protected ProxyAudioProducer(long cPtr, boolean cMemoryOwn) {
    super(tinyWRAPJNI.ProxyAudioProducer_SWIGUpcast(cPtr), cMemoryOwn);
    swigCPtr = cPtr;
  }

  protected static long getCPtr(ProxyAudioProducer obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        tinyWRAPJNI.delete_ProxyAudioProducer(swigCPtr);
      }
      swigCPtr = 0;
    }
    super.delete();
  }

  public boolean setActualSndCardRecordParams(int nPtime, int nRate, int nChannels) {
    return tinyWRAPJNI.ProxyAudioProducer_setActualSndCardRecordParams(swigCPtr, this, nPtime, nRate, nChannels);
  }

  public boolean setPushBuffer(java.nio.ByteBuffer pPushBufferPtr, long nPushBufferSize, boolean bUsePushCallback) {
    return tinyWRAPJNI.ProxyAudioProducer_setPushBuffer__SWIG_0(swigCPtr, this, pPushBufferPtr, nPushBufferSize, bUsePushCallback);
  }

  public boolean setPushBuffer(java.nio.ByteBuffer pPushBufferPtr, long nPushBufferSize) {
    return tinyWRAPJNI.ProxyAudioProducer_setPushBuffer__SWIG_1(swigCPtr, this, pPushBufferPtr, nPushBufferSize);
  }

  public int push(java.nio.ByteBuffer pBuffer, long nSize) {
    return tinyWRAPJNI.ProxyAudioProducer_push__SWIG_0(swigCPtr, this, pBuffer, nSize);
  }

  public int push(java.nio.ByteBuffer pBuffer) {
    return tinyWRAPJNI.ProxyAudioProducer_push__SWIG_1(swigCPtr, this, pBuffer);
  }

  public int push() {
    return tinyWRAPJNI.ProxyAudioProducer_push__SWIG_2(swigCPtr, this);
  }

  public boolean setGain(long nGain) {
    return tinyWRAPJNI.ProxyAudioProducer_setGain(swigCPtr, this, nGain);
  }

  public long getGain() {
    return tinyWRAPJNI.ProxyAudioProducer_getGain(swigCPtr, this);
  }

  public void setCallback(ProxyAudioProducerCallback pCallback) {
    tinyWRAPJNI.ProxyAudioProducer_setCallback(swigCPtr, this, ProxyAudioProducerCallback.getCPtr(pCallback), pCallback);
  }

  public java.math.BigInteger getMediaSessionId() {
    return tinyWRAPJNI.ProxyAudioProducer_getMediaSessionId(swigCPtr, this);
  }

  public static boolean registerPlugin() {
    return tinyWRAPJNI.ProxyAudioProducer_registerPlugin();
  }

}
