/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.4
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.utils;

public class AndroidUtils {
  private long swigCPtr;
  protected boolean swigCMemOwn;

  public AndroidUtils(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  public static long getCPtr(AndroidUtils obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        utilsJNI.delete_AndroidUtils(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public AndroidUtils() {
    this(utilsJNI.new_AndroidUtils(), true);
  }

  public static java.math.BigInteger getCpuFamily() {
    return utilsJNI.AndroidUtils_getCpuFamily();
  }

  public static java.math.BigInteger getCpuFeatures() {
    return utilsJNI.AndroidUtils_getCpuFeatures();
  }


}
