/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.4
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.utils;

public class utilsJNI {
  public final static native long new_AndroidUtils();
  public final static native void delete_AndroidUtils(long jarg1);
  public final static native java.math.BigInteger AndroidUtils_getCpuFamily();
  public final static native java.math.BigInteger AndroidUtils_getCpuFeatures();
}
