/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public interface tinyWRAPConstants {
  public final static int tsip_event_code_dialog_transport_error = 702;
  public final static int tsip_event_code_dialog_global_error = 703;
  public final static int tsip_event_code_dialog_message_error = 704;
  public final static int tsip_event_code_dialog_request_incoming = 800;
  public final static int tsip_event_code_dialog_request_outgoing = 802;
  public final static int tsip_event_code_dialog_request_cancelled = 803;
  public final static int tsip_event_code_dialog_request_sent = 804;
  public final static int tsip_event_code_dialog_connecting = 900;
  public final static int tsip_event_code_dialog_connected = 901;
  public final static int tsip_event_code_dialog_terminating = 902;
  public final static int tsip_event_code_dialog_terminated = 903;
  public final static int tsip_event_code_stack_starting = 950;
  public final static int tsip_event_code_stack_started = 951;
  public final static int tsip_event_code_stack_stopping = 952;
  public final static int tsip_event_code_stack_stopped = 953;
  public final static int tsip_event_code_stack_failed_to_start = 954;
  public final static int tsip_event_code_stack_failed_to_stop = 955;
  public final static int tsip_event_code_stack_disconnected = 956;
}
