/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public class ProxyPluginMgr {
  private long swigCPtr;
  protected boolean swigCMemOwn;

  protected ProxyPluginMgr(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(ProxyPluginMgr obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        tinyWRAPJNI.delete_ProxyPluginMgr(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public static ProxyPluginMgr createInstance(ProxyPluginMgrCallback pCallback) {
    long cPtr = tinyWRAPJNI.ProxyPluginMgr_createInstance(ProxyPluginMgrCallback.getCPtr(pCallback), pCallback);
    return (cPtr == 0) ? null : new ProxyPluginMgr(cPtr, true);
  }

  public static ProxyPluginMgr getInstance() {
    long cPtr = tinyWRAPJNI.ProxyPluginMgr_getInstance();
    return (cPtr == 0) ? null : new ProxyPluginMgr(cPtr, false);
  }

  public ProxyPlugin findPlugin(java.math.BigInteger id) {
    long cPtr = tinyWRAPJNI.ProxyPluginMgr_findPlugin(swigCPtr, this, id);
    return (cPtr == 0) ? null : new ProxyPlugin(cPtr, false);
  }

  public ProxyAudioConsumer findAudioConsumer(java.math.BigInteger id) {
    long cPtr = tinyWRAPJNI.ProxyPluginMgr_findAudioConsumer(swigCPtr, this, id);
    return (cPtr == 0) ? null : new ProxyAudioConsumer(cPtr, false);
  }

  public ProxyVideoConsumer findVideoConsumer(java.math.BigInteger id) {
    long cPtr = tinyWRAPJNI.ProxyPluginMgr_findVideoConsumer(swigCPtr, this, id);
    return (cPtr == 0) ? null : new ProxyVideoConsumer(cPtr, false);
  }

  public ProxyAudioProducer findAudioProducer(java.math.BigInteger id) {
    long cPtr = tinyWRAPJNI.ProxyPluginMgr_findAudioProducer(swigCPtr, this, id);
    return (cPtr == 0) ? null : new ProxyAudioProducer(cPtr, false);
  }

  public ProxyVideoProducer findVideoProducer(java.math.BigInteger id) {
    long cPtr = tinyWRAPJNI.ProxyPluginMgr_findVideoProducer(swigCPtr, this, id);
    return (cPtr == 0) ? null : new ProxyVideoProducer(cPtr, false);
  }

}
