/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public class OptionsEvent extends SipEvent {
  private long swigCPtr;

  protected OptionsEvent(long cPtr, boolean cMemoryOwn) {
    super(tinyWRAPJNI.OptionsEvent_SWIGUpcast(cPtr), cMemoryOwn);
    swigCPtr = cPtr;
  }

  protected static long getCPtr(OptionsEvent obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        tinyWRAPJNI.delete_OptionsEvent(swigCPtr);
      }
      swigCPtr = 0;
    }
    super.delete();
  }

  public tsip_options_event_type_t getType() {
    return tsip_options_event_type_t.swigToEnum(tinyWRAPJNI.OptionsEvent_getType(swigCPtr, this));
  }

  public OptionsSession getSession() {
    long cPtr = tinyWRAPJNI.OptionsEvent_getSession(swigCPtr, this);
    return (cPtr == 0) ? null : new OptionsSession(cPtr, false);
  }

  public OptionsSession takeSessionOwnership() {
    long cPtr = tinyWRAPJNI.OptionsEvent_takeSessionOwnership(swigCPtr, this);
    return (cPtr == 0) ? null : new OptionsSession(cPtr, false);
  }

}
