/* 

*  Copyright (C) 2020, University of the Basque Country (UPV/EHU)
*
* Contact for licensing options: <licensing-mcpttclient(at)mcopenplatform(dot)com>
*       
* This file is part of MCOP MCPTT Client
*
* This is free software: you can redistribute it and/or modify it under the terms of
* the GNU General Public License as published by the Free Software Foundation, either version 3 
* of the License, or (at your option) any later version.
*       
* This is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
* See the GNU General Public License for more details.
*       
* You should have received a copy of the GNU General Public License along 
* with this program; if not, write to the Free Software Foundation, Inc., 
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
 * @file tinymcptt.h
 * @brief tinyMCPTT API.
 *
 */
#ifndef TINYMEDIA_TINYMCPTT_H
#define TINYMEDIA_TINYMCPTT_H

/* === tinyNET (tinyNET/src) === */
#include "tnet.h"

/* === tinySAK (tinySAK/src)=== */
#include "tsk.h"

//#include "tmsrp.h"

#include "tinymcptt/tmcptt_message.h"
#include "tinymcptt/tmcptt_event.h"
#include "tinymcptt/tmcptt_mbms_event.h"
//#include "tinymsrp/tmsrp_uri.h"

/*
#include "tinymcptt/packet/tmcptt_mcptt_packet.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_ack.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_deny.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_granted.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_idle.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_release.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_request.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_revoke.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_specific.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_taken.h"
#include "tinymcptt/packet/tmcptt_mcptt_packet_timestamp.h"
*/

/*
#include "tinymcptt/parsers/tmcptt_parser_message.h"
#include "tinymcptt/parsers/tmcptt_parser_uri.h"

#include "tinymcptt/session/tmcptt_config.h"
#include "tinymcptt/session/tmcptt_data.h"
#include "tinymcptt/session/tmcptt_receiver.h"
#include "tinymcptt/session/tmcptt_sender.h"
*/
#endif /* TINYMEDIA_TINYMCPTT_H */
