/*
 *

 *   Copyright (C) 2020, University of the Basque Country (UPV/EHU)
 *
 *  Contact for licensing options: <licensing-mcpttclient(at)mcopenplatform(dot)com>
 *
 *  This file is part of MCOP MCPTT Client
 *
 *  This is free software: you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by the Free Software Foundation, either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package org.doubango.ngn.services.profiles;

import android.content.Context;

import org.doubango.ngn.services.INgnBaseService;
import org.doubango.ngn.sip.NgnSipPrefrences;

import java.util.ArrayList;
import java.util.List;

public interface IMyProfilesService extends INgnBaseService {



     NgnSipPrefrences getProfile(Context context, String name,boolean forceReadPofile);

     ArrayList<String> getProfilesNames(Context context);

     boolean setProfileNow(Context context,String nameProfile);

     void setProfileNow(NgnSipPrefrences profileNow);
     NgnSipPrefrences getProfileNow(Context context);

     boolean invalidProfile(Context context);

    //Used by NGNsipService to load data from default profile.
    interface  OnSetProfileListener{
        void onSetProfile();
    }

    void setOnSetProfileListener(OnSetProfileListener mOnSetProfileListener);

    void setProfiles(List<NgnSipPrefrences> profiles);

    boolean importProfiles(String profiles,Context context);


}
