/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public enum twrap_media_type_t {
  twrap_media_none(0x00),
  twrap_media_audio(0x01),
  twrap_media_video(0x02),
  twrap_media_msrp(0x04),
  twrap_media_t140(0x08),
  twrap_media_bfcp(0x10),
  twrap_media_bfcp_audio(0x30),
  twrap_media_bfcp_video(0x50),
  twrap_media_audiovideo(0x03),
  twrap_media_audio_video(twrap_media_audiovideo),
  twrap_media_floor_control((0x01 << 27)),
  twrap_media_mcptt((0x01 << 13)),
  twrap_media_group((0x01 << 14)),
  twrap_media_audio_ptt_mcptt((0x01 << 13)|(0x01)),
  twrap_media_audio_ptt_mcptt_with_floor_control((0x01 << 13)|(0x01) | twrap_media_floor_control.swigValue),
  twrap_media_audio_ptt_group_mcptt((0x01 << 13)|(0x01)|(0x01 << 14)),
  twrap_media_audio_ptt_group_mcptt_with_floor_control((0x01 << 13)|(0x01)|(0x01 << 14) | twrap_media_floor_control.swigValue),
  twrap_media_location((0x01 << 15)),
  twrap_media_mcptt_location((0x01 << 15)|(0x01 << 13)),
  twrap_media_affiliation((0x01 << 16)),
  twrap_media_mcptt_affiliation((0x01 << 16)|(0x01 << 13)),
  twrap_media_mbms((0x01 << 17)),
  twrap_media_mcptt_mbms((0x01 << 17)|(0x01 << 13)),
  twrap_media_authentication((0x01 << 18)),
  twrap_media_mcptt_authentication((0x01 << 18)|(0x01 << 13)),
  twrap_media_emergency((0x01 << 19)),
  twrap_media_alert((0x01 << 20)),
  twrap_media_imminentperil((0x01 << 21)),
  twrap_media_mcptt_emergence((0x01 << 19)|(0x01 << 13)|(0x01)),
  twrap_media_mcptt_emergence_with_floor_control((0x01 << 19)|(0x01 << 13)|(0x01) | twrap_media_floor_control.swigValue),
  twrap_media_mcptt_group_emergence((0x01 << 19)|(0x01 << 13)|(0x01)|(0x01 << 14)),
  twrap_media_mcptt_group_emergence_with_floor_control((0x01 << 19)|(0x01 << 13)|(0x01)|(0x01 << 14) | twrap_media_floor_control.swigValue),
  twrap_media_mcptt_alert((0x01 << 20)|(0x01 << 13)|(0x01)),
  twrap_media_mcptt_alert_with_floor_control((0x01 << 20)|(0x01 << 13)|(0x01) | twrap_media_floor_control.swigValue),
  twrap_media_mcptt_group_alert((0x01 << 20)|(0x01 << 13)|(0x01)|(0x01 << 14)),
  twrap_media_mcptt_group_alert_with_floor_control((0x01 << 20)|(0x01 << 13)|(0x01)|(0x01 << 14) | twrap_media_floor_control.swigValue),
  twrap_media_mcptt_imminentperil((0x01 << 21)|(0x01 << 13)|(0x01)),
  twrap_media_mcptt_imminentperil_with_floor_control((0x01 << 21)|(0x01 << 13)|(0x01) | twrap_media_floor_control.swigValue),
  twrap_media_mcptt_group_imminentperil((0x01 << 21)|(0x01 << 13)|(0x01)|(0x01 << 14)),
  twrap_media_mcptt_group_imminentperil_with_floor_control((0x01 << 21)|(0x01 << 13)|(0x01)|(0x01 << 14) | twrap_media_floor_control.swigValue),
  twrap_media_cms((0x01 << 22)),
  twrap_media_mcptt_cms(twrap_media_mcptt.swigValue|twrap_media_cms.swigValue),
  twrap_media_gms((0x01 << 23)),
  twrap_media_mcptt_gms(twrap_media_mcptt.swigValue|twrap_media_gms.swigValue),
  twrap_media_mcptt_chat((0x01 << 24)),
  twrap_media_audio_ptt_chat_mcptt(twrap_media_audio_ptt_mcptt.swigValue|twrap_media_mcptt_chat.swigValue),
  twrap_media_audio_ptt_chat_group_mcptt(twrap_media_audio_ptt_mcptt.swigValue|twrap_media_mcptt_chat.swigValue|twrap_media_group.swigValue),
  twrap_media_audio_ptt_chat_group_mcptt_with_floor_control(twrap_media_audio_ptt_mcptt.swigValue|twrap_media_mcptt_chat.swigValue|twrap_media_group.swigValue | twrap_media_floor_control.swigValue);

  public final int swigValue() {
    return swigValue;
  }

  public static twrap_media_type_t swigToEnum(int swigValue) {
    twrap_media_type_t[] swigValues = twrap_media_type_t.class.getEnumConstants();
    if (swigValue < swigValues.length && swigValue >= 0 && swigValues[swigValue].swigValue == swigValue)
      return swigValues[swigValue];
    for (twrap_media_type_t swigEnum : swigValues)
      if (swigEnum.swigValue == swigValue)
        return swigEnum;
    throw new IllegalArgumentException("No enum " + twrap_media_type_t.class + " with value " + swigValue);
  }

  @SuppressWarnings("unused")
  private twrap_media_type_t() {
    this.swigValue = SwigNext.next++;
  }

  @SuppressWarnings("unused")
  private twrap_media_type_t(int swigValue) {
    this.swigValue = swigValue;
    SwigNext.next = swigValue+1;
  }

  @SuppressWarnings("unused")
  private twrap_media_type_t(twrap_media_type_t swigEnum) {
    this.swigValue = swigEnum.swigValue;
    SwigNext.next = this.swigValue+1;
  }

  private final int swigValue;

  private static class SwigNext {
    private static int next = 0;
  }
}

