/*
 *
 *   Copyright (C) 2020, University of the Basque Country (UPV/EHU)
 *
 *  Contact for licensing options: <licensing-mcpttclient(at)mcopenplatform(dot)com>
 *
 *  This file is part of MCOP MCPTT Client
 *
 *  This is free software: you can redistribute it and/or modify it under the terms of
 *  the GNU General Public License as published by the Free Software Foundation, either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */



package org.doubango.ngn.datatype.mo; 
 /**
 * This object contains factory methods for each 
 * Java content interface and Java element interface 
 * generated in the org.doubango.ngn.datatype.mo package. 
 * <p>An ObjectFactory allows you to programatically 
 * construct new instances of the Java representation 
 * for XML content. The Java representation of XML 
 * content can consist of schema derived interfaces 
 * and classes representing the binding of schema 
 * type definitions, element declarations and model 
 * groups.  Factory methods for each of these are 
 * provided in this class.
 * 
 */
public class ObjectFactory {


    /**
     * Create a new ObjectFactory that can be used to create new instances of schema derived classes for package: org.doubango.ngn.datatype.mo
     * 
     */
    public ObjectFactory() {
    }

    /**
     * Create an instance of {@link Date }
     * 
     */
    public Date createDate() {
        return new Date();
    }

    /**
     * Create an instance of {@link Add }
     * 
     */
    public Add createAdd() {
        return new Add();
    }

    /**
     * Create an instance of {@link Delete }
     * 
     */
    public Delete createDelete() {
        return new Delete();
    }

    /**
     * Create an instance of {@link RTProperties }
     * 
     */
    public RTProperties createRTProperties() {
        return new RTProperties();
    }

    /**
     * Create an instance of {@link Format }
     * 
     */
    public Format createFormat() {
        return new Format();
    }

    /**
     * Create an instance of {@link Type }
     * 
     */
    public Type createType() {
        return new Type();
    }

    /**
     * Create an instance of {@link Bin }
     * 
     */
    public Bin createBin() {
        return new Bin();
    }

    /**
     * Create an instance of {@link Node }
     * 
     */
    public Node createNode() {
        return new Node();
    }

    /**
     * Create an instance of {@link DFProperties }
     * 
     */
    public DFProperties createDFProperties() {
        return new DFProperties();
    }

    /**
     * Create an instance of {@link Value }
     * 
     */
    public Value createValue() {
        return new Value();
    }

    /**
     * Create an instance of {@link Float }
     * 
     */
    public Float createFloat() {
        return new Float();
    }

    /**
     * Create an instance of {@link AccessType }
     * 
     */
    public AccessType createAccessType() {
        return new AccessType();
    }

    /**
     * Create an instance of {@link DFFormat }
     * 
     */
    public DFFormat createDFFormat() {
        return new DFFormat();
    }

    /**
     * Create an instance of {@link Occurrence }
     * 
     */
    public Occurrence createOccurrence() {
        return new Occurrence();
    }

    /**
     * Create an instance of {@link Scope }
     * 
     */
    public Scope createScope() {
        return new Scope();
    }

    /**
     * Create an instance of {@link DFType }
     * 
     */
    public DFType createDFType() {
        return new DFType();
    }

    /**
     * Create an instance of {@link CaseSense }
     * 
     */
    public CaseSense createCaseSense() {
        return new CaseSense();
    }

    /**
     * Create an instance of {@link MIME }
     * 
     */
    public MIME createMIME() {
        return new MIME();
    }

    /**
     * Create an instance of {@link B64 }
     * 
     */
    public B64 createB64() {
        return new B64();
    }

    /**
     * Create an instance of {@link Bool }
     * 
     */
    public Bool createBool() {
        return new Bool();
    }

    /**
     * Create an instance of {@link Chr }
     * 
     */
    public Chr createChr() {
        return new Chr();
    }

    /**
     * Create an instance of {@link Int }
     * 
     */
    public Int createInt() {
        return new Int();
    }

    /**
     * Create an instance of {@link Nodie }
     * 
     */
    public Nodie createNodie() {
        return new Nodie();
    }

    /**
     * Create an instance of {@link Null }
     * 
     */
    public Null createNull() {
        return new Null();
    }

    /**
     * Create an instance of {@link Xml }
     * 
     */
    public Xml createXml() {
        return new Xml();
    }

    /**
     * Create an instance of {@link Time }
     * 
     */
    public Time createTime() {
        return new Time();
    }

    /**
     * Create an instance of {@link Exec }
     * 
     */
    public Exec createExec() {
        return new Exec();
    }

    /**
     * Create an instance of {@link One }
     * 
     */
    public One createOne() {
        return new One();
    }

    /**
     * Create an instance of {@link ZeroOrOne }
     * 
     */
    public ZeroOrOne createZeroOrOne() {
        return new ZeroOrOne();
    }

    /**
     * Create an instance of {@link ZeroOrMore }
     * 
     */
    public ZeroOrMore createZeroOrMore() {
        return new ZeroOrMore();
    }

    /**
     * Create an instance of {@link OneOrMore }
     * 
     */
    public OneOrMore createOneOrMore() {
        return new OneOrMore();
    }

    /**
     * Create an instance of {@link ZeroOrN }
     * 
     */
    public ZeroOrN createZeroOrN() {
        return new ZeroOrN();
    }

    /**
     * Create an instance of {@link OneOrN }
     * 
     */
    public OneOrN createOneOrN() {
        return new OneOrN();
    }

    /**
     * Create an instance of {@link Copy }
     * 
     */
    public Copy createCopy() {
        return new Copy();
    }

    /**
     * Create an instance of {@link Get }
     * 
     */
    public Get createGet() {
        return new Get();
    }

    /**
     * Create an instance of {@link Replace }
     * 
     */
    public Replace createReplace() {
        return new Replace();
    }

    /**
     * Create an instance of {@link CIS }
     * 
     */
    public CIS createCIS() {
        return new CIS();
    }

    /**
     * Create an instance of {@link DDFName }
     * 
     */
    public DDFName createDDFName() {
        return new DDFName();
    }

    /**
     * Create an instance of {@link Dynamic }
     * 
     */
    public Dynamic createDynamic() {
        return new Dynamic();
    }

    /**
     * Create an instance of {@link Permanent }
     * 
     */
    public Permanent createPermanent() {
        return new Permanent();
    }

    /**
     * Create an instance of {@link CS }
     * 
     */
    public CS createCS() {
        return new CS();
    }

    /**
     * Create an instance of {@link MgmtTree }
     * 
     */
    public MgmtTree createMgmtTree() {
        return new MgmtTree();
    }

}
