///* Copyright (C) 2014 Mamadou DIOP.
//*
//* This file is part of Open Source Doubango Framework.
//*
//* DOUBANGO is free software: you can redistribute it and/or modify
//* it under the terms of the GNU General Public License as published by
//* the Free Software Foundation, either version 3 of the License, or
//* (at your option) any later version.
//*
//* DOUBANGO is distributed in the hope that it will be useful,
//* but WITHOUT ANY WARRANTY; without even the implied warranty of
//* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//* GNU Lesser General Public License for more details.
//*
//* You should have received a copy of the GNU General Public License
//* along with DOUBANGO.
//*
//*/
//#define TNET_TURN_ATTR_H
//
//#include "tinynet_config.h"
//#include "stun/tnet_stun_types.h"
//
//#include "tsk_object.h"
//#include "tsk_list.h"
//
//TNET_BEGIN_DECLS
//
//TNET_END_DECLS
//
