/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public class SipCallback {
  private long swigCPtr;
  protected boolean swigCMemOwn;

  protected SipCallback(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(SipCallback obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        tinyWRAPJNI.delete_SipCallback(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  protected void swigDirectorDisconnect() {
    swigCMemOwn = false;
    delete();
  }

  public void swigReleaseOwnership() {
    swigCMemOwn = false;
    tinyWRAPJNI.SipCallback_change_ownership(this, swigCPtr, false);
  }

  public void swigTakeOwnership() {
    swigCMemOwn = true;
    tinyWRAPJNI.SipCallback_change_ownership(this, swigCPtr, true);
  }

  public SipCallback() {
    this(tinyWRAPJNI.new_SipCallback(), true);
    tinyWRAPJNI.SipCallback_director_connect(this, swigCPtr, swigCMemOwn, true);
  }

  public int OnDialogEvent(DialogEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnDialogEvent(swigCPtr, this, DialogEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnDialogEventSwigExplicitSipCallback(swigCPtr, this, DialogEvent.getCPtr(e), e);
  }

  public int OnStackEvent(StackEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnStackEvent(swigCPtr, this, StackEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnStackEventSwigExplicitSipCallback(swigCPtr, this, StackEvent.getCPtr(e), e);
  }

  public int OnInviteEvent(InviteEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnInviteEvent(swigCPtr, this, InviteEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnInviteEventSwigExplicitSipCallback(swigCPtr, this, InviteEvent.getCPtr(e), e);
  }

  public int OnMessagingEvent(MessagingEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnMessagingEvent(swigCPtr, this, MessagingEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnMessagingEventSwigExplicitSipCallback(swigCPtr, this, MessagingEvent.getCPtr(e), e);
  }

  public int OnMessagingAffiliationEvent(MessagingAffiliationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnMessagingAffiliationEvent(swigCPtr, this, MessagingAffiliationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnMessagingAffiliationEventSwigExplicitSipCallback(swigCPtr, this, MessagingAffiliationEvent.getCPtr(e), e);
  }

  public int OnMessagingMbmsEvent(MessagingMbmsEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnMessagingMbmsEvent(swigCPtr, this, MessagingMbmsEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnMessagingMbmsEventSwigExplicitSipCallback(swigCPtr, this, MessagingMbmsEvent.getCPtr(e), e);
  }

  public int OnMessagingLocationEvent(MessagingLocationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnMessagingLocationEvent(swigCPtr, this, MessagingLocationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnMessagingLocationEventSwigExplicitSipCallback(swigCPtr, this, MessagingLocationEvent.getCPtr(e), e);
  }

  public int OnInfoEvent(InfoEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnInfoEvent(swigCPtr, this, InfoEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnInfoEventSwigExplicitSipCallback(swigCPtr, this, InfoEvent.getCPtr(e), e);
  }

  public int OnOptionsEvent(OptionsEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnOptionsEvent(swigCPtr, this, OptionsEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnOptionsEventSwigExplicitSipCallback(swigCPtr, this, OptionsEvent.getCPtr(e), e);
  }

  public int OnPublicationEvent(PublicationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnPublicationEvent(swigCPtr, this, PublicationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnPublicationEventSwigExplicitSipCallback(swigCPtr, this, PublicationEvent.getCPtr(e), e);
  }

  public int OnPublicationAffiliationEvent(PublicationAffiliationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnPublicationAffiliationEvent(swigCPtr, this, PublicationAffiliationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnPublicationAffiliationEventSwigExplicitSipCallback(swigCPtr, this, PublicationAffiliationEvent.getCPtr(e), e);
  }

  public int OnPublicationAuthenticationEvent(PublicationAuthenticationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnPublicationAuthenticationEvent(swigCPtr, this, PublicationAuthenticationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnPublicationAuthenticationEventSwigExplicitSipCallback(swigCPtr, this, PublicationAuthenticationEvent.getCPtr(e), e);
  }

  public int OnRegistrationAuthenticationEvent(RegistrationAuthenticationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnRegistrationAuthenticationEvent(swigCPtr, this, RegistrationAuthenticationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnRegistrationAuthenticationEventSwigExplicitSipCallback(swigCPtr, this, RegistrationAuthenticationEvent.getCPtr(e), e);
  }

  public int OnRegistrationEvent(RegistrationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnRegistrationEvent(swigCPtr, this, RegistrationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnRegistrationEventSwigExplicitSipCallback(swigCPtr, this, RegistrationEvent.getCPtr(e), e);
  }

  public int OnSubscriptionEvent(SubscriptionEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnSubscriptionEvent(swigCPtr, this, SubscriptionEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnSubscriptionEventSwigExplicitSipCallback(swigCPtr, this, SubscriptionEvent.getCPtr(e), e);
  }

  public int OnSubscriptionAffiliationEvent(SubscriptionAffiliationEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnSubscriptionAffiliationEvent(swigCPtr, this, SubscriptionAffiliationEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnSubscriptionAffiliationEventSwigExplicitSipCallback(swigCPtr, this, SubscriptionAffiliationEvent.getCPtr(e), e);
  }

  public int OnSubscriptionCMSEvent(SubscriptionCMSEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnSubscriptionCMSEvent(swigCPtr, this, SubscriptionCMSEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnSubscriptionCMSEventSwigExplicitSipCallback(swigCPtr, this, SubscriptionCMSEvent.getCPtr(e), e);
  }

  public int OnSubscriptionGMSEvent(SubscriptionGMSEvent e) {
    return (getClass() == SipCallback.class) ? tinyWRAPJNI.SipCallback_OnSubscriptionGMSEvent(swigCPtr, this, SubscriptionGMSEvent.getCPtr(e), e) : tinyWRAPJNI.SipCallback_OnSubscriptionGMSEventSwigExplicitSipCallback(swigCPtr, this, SubscriptionGMSEvent.getCPtr(e), e);
  }

}
