/* config.h.  Generated from config.h.in by configure.  */
/* config.h.in.  Generated from configure.ac by autoheader.  */

/* fatal debug */
/* #undef DEBUG_LEVEL */

/* Define if <alsa/asoundlib.h> header exist */
#define HAVE_ALSA_ASOUNDLIB_H 0

/* Checks if the installed libsrtp version support append_salt_to_key()
   function */
#define HAVE_APPEND_SALT_TO_KEY 1

/* Define to 1 if you have the <arpa/inet.h> header file. */
#define HAVE_ARPA_INET_H 1

/* Define to 1 if we have the `clock_gettime' function. */
/* #undef HAVE_CLOCK_GETTIME */

/* Define to 1 if you have the `closedir' function. */
#define HAVE_CLOSEDIR 1

/* Define to 1 if you have the <dirent.h> header file. */
#define HAVE_DIRENT_H 1

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define if <dns.h> header exist */
#define HAVE_DNS_H 0

/* Define to 1 if you have the <fcntl.h> header file. */
#define HAVE_FCNTL_H 1

/* HAVE_FFMPEG */
#define HAVE_FFMPEG 1

/* HAVE_G729 */
/* #undef HAVE_G729 */

/* Define to 1 if you have the <g729b/ld8a.h> header file. */
#define HAVE_G729B_LD8A_H 1

/* Define to 1 if you have the <g729b/typedef.h> header file. */
#define HAVE_G729B_TYPEDEF_H 1

/* Define to 1 if you have the `getdtablesize' function. */
#define HAVE_GETDTABLESIZE 1

/* Define to 1 if you have the 'getifaddrs' function */
/* #undef HAVE_GETIFADDRS */

/* Define to 1 if you have the `getpid' function. */
#define HAVE_GETPID 1

/* Define to 1 if you have the `getrlimit' function. */
#define HAVE_GETRLIMIT 1

/* Define to 1 if you have the <gsm.h> header file. */
#define HAVE_GSM_H 1

/* Define if <ifaddrs.h> header exist */
#define HAVE_IFADDRS_H 0

/* HAVE_ILBC */
#define HAVE_ILBC 1

/* Define to 1 if you have the <ilbc/iLBC_decode.h> header file. */
#define HAVE_ILBC_ILBC_DECODE_H 1

/* Define to 1 if you have the <ilbc/iLBC_define.h> header file. */
#define HAVE_ILBC_ILBC_DEFINE_H 1

/* Define to 1 if you have the <ilbc/iLBC_encode.h> header file. */
#define HAVE_ILBC_ILBC_ENCODE_H 1

/* Define to 1 if you have the `inet_ntop' function. */
#define HAVE_INET_NTOP 1

/* Define to 1 if you have the `inet_pton' function. */
#define HAVE_INET_PTON 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the <jerror.h> header file. */
/* #undef HAVE_JERROR_H */

/* Define to 1 if you have the <jpeglib.h> header file. */
/* #undef HAVE_JPEGLIB_H */

/* Define to 1 if you have the <libavcodec/avcodec.h> header file. */
#define HAVE_LIBAVCODEC_AVCODEC_H 1

/* Define to 1 if you have the <libavutil/avutil.h> header file. */
#define HAVE_LIBAVUTIL_AVUTIL_H 1

/* HAVE_LIBGSM */
#define HAVE_LIBGSM 1

/* HAVE_LIBJPEG */
/* #undef HAVE_LIBJPEG */

/* HAVE_LIBOPUS */
#define HAVE_LIBOPUS 1

/* Define to 1 if you have the <libswscale/swscale.h> header file. */
#define HAVE_LIBSWSCALE_SWSCALE_H 1

/* HAVE_LIBVPX */
#define HAVE_LIBVPX 1

/* HAVE_LIBXML2 */
#define HAVE_LIBXML2 1

/* Define to 1 if you have the <libxml/parser.h> header file. */
#define HAVE_LIBXML_PARSER_H 1

/* Define to 1 if you have the <libxml/tree.h> header file. */
#define HAVE_LIBXML_TREE_H 1

/* Define to 1 if you have the <libxml/xpath.h> header file. */
#define HAVE_LIBXML_XPATH_H 1

/* HAVE_LIBYUV */
#define HAVE_LIBYUV 1

/* Define to 1 if you have the <libyuv/libyuv.h> header file. */
#define HAVE_LIBYUV_LIBYUV_H 1

/* HAVE_LIB_SPEEX */
#define HAVE_LIB_SPEEX 1

/* Define if <linux/soundcard.h> header exist */
#define HAVE_LINUX_SOUNDCARD_H 1

/* Define if <linux/videodev2.h> header exist */
#define HAVE_LINUX_VIDEODEV2_H 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define if <netpacket/packet.h> header exist */
#define HAVE_NETPACKET_PACKET_H 1

/* Define if <net/if_dl.h> header exist */
#define HAVE_NET_IF_DL_H 0

/* Define if <net/if_types> header exist */
#define HAVE_NET_IF_TYPES_H 1

/* Define if <net/route.h> header exist */
#define HAVE_NET_ROUTE_H 1

/* HAVE_OPENCORE_AMR */
/* #undef HAVE_OPENCORE_AMR */

/* Define to 1 if you have the <opencore-amrnb/interf_dec.h> header file. */
/* #undef HAVE_OPENCORE_AMRNB_INTERF_DEC_H */

/* Define to 1 if you have the <opencore-amrnb/interf_enc.h> header file. */
/* #undef HAVE_OPENCORE_AMRNB_INTERF_ENC_H */

/* Define to 1 if you have the `opendir' function. */
#define HAVE_OPENDIR 1

/* HAVE_OPENH264 */
#define HAVE_OPENH264 1

/* HAVE_OPENSSL */
#define HAVE_OPENSSL 1

/* HAVE_OPENSSL_DTLS */
#define HAVE_OPENSSL_DTLS 1

/* HAVE_OPENSSL_DTLS_SRTP */
#define HAVE_OPENSSL_DTLS_SRTP 1

/* Define to 1 if you have the <openssl/ssl.h> header file. */
#define HAVE_OPENSSL_SSL_H 1

/* Define to 1 if you have the <opus/opus.h> header file. */
#define HAVE_OPUS_OPUS_H 1

/* Define to 1 if you have the `poll' function. */
#define HAVE_POLL 1

/* Define to 1 if you have the <poll.h> header file. */
#define HAVE_POLL_H 1

/* Define to 1 if you have the `setrlimit' function. */
#define HAVE_SETRLIMIT 1

/* HAVE_SPEEX_DENOISE */
#define HAVE_SPEEX_DENOISE 1

/* HAVE_SPEEX_DSP */
#define HAVE_SPEEX_DSP 1

/* HAVE_SPEEX_JB */
#define HAVE_SPEEX_JB 1

/* HAVE_SPEEX_RESAMPLER */
#define HAVE_SPEEX_RESAMPLER 1

/* Define to 1 if you have the <speex/speex_echo.h> header file. */
#define HAVE_SPEEX_SPEEX_ECHO_H 1

/* Define to 1 if you have the <speex/speex.h> header file. */
#define HAVE_SPEEX_SPEEX_H 1

/* Define to 1 if you have the <speex/speex_jitter.h> header file. */
#define HAVE_SPEEX_SPEEX_JITTER_H 1

/* Define to 1 if you have the <speex/speex_preprocess.h> header file. */
#define HAVE_SPEEX_SPEEX_PREPROCESS_H 1

/* Define to 1 if you have the <speex/speex_resampler.h> header file. */
#define HAVE_SPEEX_SPEEX_RESAMPLER_H 1

/* HAVE_SRTP */
#define HAVE_SRTP 1

/* Checks if the installed libsrtp version support
   srtp_profile_get_master_key_length() function */
#define HAVE_SRTP_PROFILE_GET_MASTER_KEY_LENGTH 1

/* Checks if the installed libsrtp version support
   srtp_profile_get_master_salt_length() function */
#define HAVE_SRTP_PROFILE_GET_MASTER_SALT_LENGTH 1

/* Define to 1 if you have the <srtp/srtp.h> header file. */
#define HAVE_SRTP_SRTP_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the `stricmp' function. */
/* #undef HAVE_STRICMP */

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the `strnicmp' function. */
/* #undef HAVE_STRNICMP */

/* Define to 1 if you have the `strtok_r' function. */
#define HAVE_STRTOK_R 1

/* Define to 1 if you have the `strtok_s' function. */
/* #undef HAVE_STRTOK_S */

/* Define to 1 if the system has the type `struct dirent'. */
#define HAVE_STRUCT_DIRENT 1

/* Define to 1 if the system has the type `struct rlimit'. */
#define HAVE_STRUCT_RLIMIT 1

/* Define to 1 if the system has the type `struct rt_metrics'. */
/* #undef HAVE_STRUCT_RT_METRICS */

/* Define to 1 if the system has the type `struct sockaddr_dl'. */
/* #undef HAVE_STRUCT_SOCKADDR_DL */

/* Define to 1 if you have the <sys/param.h> header file. */
#define HAVE_SYS_PARAM_H 1

/* Define to 1 if you have the <sys/resource.h> header file. */
#define HAVE_SYS_RESOURCE_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to 1 if you have the <vpx/vpx_decoder.h> header file. */
#define HAVE_VPX_VPX_DECODER_H 1

/* Define to 1 if you have the <vpx/vpx_encoder.h> header file. */
#define HAVE_VPX_VPX_ENCODER_H 1

/* HAVE_WEBRTC */
#define HAVE_WEBRTC 1

/* HAVE_WEBRTC_DENOISE */
#define HAVE_WEBRTC_DENOISE 1

/* Define to 1 if you have the <webrtc/echo_cancellation.h> header file. */
/* #undef HAVE_WEBRTC_ECHO_CANCELLATION_H */

/* Define to 1 if you have the <webrtc/echo_control_mobile.h> header file. */
#define HAVE_WEBRTC_ECHO_CONTROL_MOBILE_H 1

/* Define to 1 if you have the <webrtc/noise_suppression.h> header file. */
/* #undef HAVE_WEBRTC_NOISE_SUPPRESSION_H */

/* Define to 1 if you have the <webrtc/noise_suppression_x.h> header file. */
#define HAVE_WEBRTC_NOISE_SUPPRESSION_X_H 1

/* Define to 1 if you have the <wels/codec_api.h> header file. */
#define HAVE_WELS_CODEC_API_H 1

/* Define to 1 if you have the <wels/codec_app_def.h> header file. */
#define HAVE_WELS_CODEC_APP_DEF_H 1

/* Check whether __sync_fetch_and_add is defined */
#define HAVE___SYNC_FETCH_AND_ADD 1

/* Check whether __sync_fetch_and_sub is defined */
#define HAVE___SYNC_FETCH_AND_SUB 1

/* Define to the sub-directory in which libtool stores uninstalled libraries.
   */
#define LT_OBJDIR ".libs/"

/* Name of package */
#define PACKAGE "libdoubango"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "doubango(at)googlegroups(dot)com"

/* Define to the full name of this package. */
#define PACKAGE_NAME "libdoubango"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "libdoubango 2.0.1200"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "libdoubango"

/* Define to the home page for this package. */
#define PACKAGE_URL ""

/* Define to the version of this package. */
#define PACKAGE_VERSION "2.0.1200"

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Define if sockaddr.sa_len exists */
#define TNET_HAVE_SA_LEN 0

/* Define if sockaddr_storage.ss_len exists */
#define TNET_HAVE_SS_LEN 0

/* Check whether PTHREAD_MUTEX_RECURSIVE_NP is defined */
#define TSK_RECURSIVE_MUTEXATTR PTHREAD_MUTEX_RECURSIVE

/* Setting USE_POLL to 1 for backward compatibility */
#define USE_POLL 1

/* Version number of package */
#define VERSION "2.0.1200"
