/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.10
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.doubango.tinyWRAP;

public class tinyWRAPJNI {
  public final static native long new_DDebugCallback();
  public final static native void delete_DDebugCallback(long jarg1);
  public final static native int DDebugCallback_OnDebugTest(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugTestSwigExplicitDDebugCallback(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugInfo(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugInfoSwigExplicitDDebugCallback(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugWarn(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugWarnSwigExplicitDDebugCallback(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugError(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugErrorSwigExplicitDDebugCallback(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugFatal(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native int DDebugCallback_OnDebugFatalSwigExplicitDDebugCallback(long jarg1, DDebugCallback jarg1_, String jarg2);
  public final static native void DDebugCallback_director_connect(DDebugCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void DDebugCallback_change_ownership(DDebugCallback obj, long cptr, boolean take_or_release);
  public final static native long new_DRegisterCallback();
  public final static native void delete_DRegisterCallback(long jarg1);
  public final static native int DRegisterCallback_onAuthRegister(long jarg1, DRegisterCallback jarg1_, String jarg2);
  public final static native int DRegisterCallback_onAuthRegisterSwigExplicitDRegisterCallback(long jarg1, DRegisterCallback jarg1_, String jarg2);
  public final static native void DRegisterCallback_director_connect(DRegisterCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void DRegisterCallback_change_ownership(DRegisterCallback obj, long cptr, boolean take_or_release);
  public final static native long new_AudioResampler(long jarg1, long jarg2, long jarg3, long jarg4, long jarg5);
  public final static native void delete_AudioResampler(long jarg1);
  public final static native boolean AudioResampler_isValid(long jarg1, AudioResampler jarg1_);
  public final static native long AudioResampler_getOutputRequiredSizeInShort(long jarg1, AudioResampler jarg1_);
  public final static native long AudioResampler_getInputRequiredSizeInShort(long jarg1, AudioResampler jarg1_);
  public final static native long AudioResampler_process(long jarg1, AudioResampler jarg1_, java.nio.ByteBuffer jarg2, long jarg3, java.nio.ByteBuffer jarg4, long jarg5);
  public final static native long new_ActionConfig();
  public final static native void delete_ActionConfig(long jarg1);
  public final static native boolean ActionConfig_addHeader(long jarg1, ActionConfig jarg1_, String jarg2, String jarg3);
  public final static native boolean ActionConfig_addPayload(long jarg1, ActionConfig jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean ActionConfig_setActiveMedia(long jarg1, ActionConfig jarg1_, int jarg2);
  public final static native long ActionConfig_setResponseLine(long jarg1, ActionConfig jarg1_, short jarg2, String jarg3);
  public final static native long ActionConfig_setMediaString(long jarg1, ActionConfig jarg1_, int jarg2, String jarg3, String jarg4);
  public final static native long ActionConfig_setMediaInt(long jarg1, ActionConfig jarg1_, int jarg2, String jarg3, int jarg4);
  public final static native void delete_Codec(long jarg1);
  public final static native int Codec_getMediaType(long jarg1, Codec jarg1_);
  public final static native String Codec_getName(long jarg1, Codec jarg1_);
  public final static native String Codec_getDescription(long jarg1, Codec jarg1_);
  public final static native String Codec_getNegFormat(long jarg1, Codec jarg1_);
  public final static native int Codec_getAudioSamplingRate(long jarg1, Codec jarg1_);
  public final static native int Codec_getAudioChannels(long jarg1, Codec jarg1_);
  public final static native int Codec_getAudioPTime(long jarg1, Codec jarg1_);
  public final static native void delete_MediaSessionMgr(long jarg1);
  public final static native boolean MediaSessionMgr_sessionSetInt32(long jarg1, MediaSessionMgr jarg1_, int jarg2, String jarg3, int jarg4);
  public final static native int MediaSessionMgr_sessionGetInt32(long jarg1, MediaSessionMgr jarg1_, int jarg2, String jarg3);
  public final static native boolean MediaSessionMgr_consumerSetInt32(long jarg1, MediaSessionMgr jarg1_, int jarg2, String jarg3, int jarg4);
  public final static native boolean MediaSessionMgr_consumerSetInt64(long jarg1, MediaSessionMgr jarg1_, int jarg2, String jarg3, long jarg4);
  public final static native boolean MediaSessionMgr_producerSetInt32(long jarg1, MediaSessionMgr jarg1_, int jarg2, String jarg3, int jarg4);
  public final static native boolean MediaSessionMgr_producerSetInt64(long jarg1, MediaSessionMgr jarg1_, int jarg2, String jarg3, long jarg4);
  public final static native long MediaSessionMgr_producerGetCodec(long jarg1, MediaSessionMgr jarg1_, int jarg2);
  public final static native int MediaSessionMgr_sessionHold(long jarg1, MediaSessionMgr jarg1_, int jarg2);
  public final static native int MediaSessionMgr_sessionResume(long jarg1, MediaSessionMgr jarg1_, int jarg2);
  public final static native long MediaSessionMgr_findProxyPluginConsumer(long jarg1, MediaSessionMgr jarg1_, int jarg2);
  public final static native long MediaSessionMgr_findProxyPluginProducer(long jarg1, MediaSessionMgr jarg1_, int jarg2);
  public final static native long MediaSessionMgr_registerAudioPluginFromFile(String jarg1);
  public final static native long MediaSessionMgr_registerAudioPluginOpenSLES();
  public final static native java.math.BigInteger MediaSessionMgr_getSessionId(long jarg1, MediaSessionMgr jarg1_, int jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetProfile(int jarg1);
  public final static native int MediaSessionMgr_defaultsGetProfile();
  public final static native boolean MediaSessionMgr_defaultsSetBandwidthLevel(int jarg1);
  public final static native int MediaSessionMgr_defaultsGetBandwidthLevel();
  public final static native boolean MediaSessionMgr_defaultsSetCongestionCtrlEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetVideoMotionRank(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetVideoFps(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetBandwidthVideoUploadMax(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetBandwidthVideoDownloadMax(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetPrefVideoSize(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetJbMargin(long jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetJbMaxLateRate(long jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetEchoTail(long jarg1);
  public final static native long MediaSessionMgr_defaultsGetEchoTail();
  public final static native boolean MediaSessionMgr_defaultsSetEchoSkew(long jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetEchoSuppEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetEchoSuppEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetAgcEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetAgcEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetAgcLevel(float jarg1);
  public final static native float MediaSessionMgr_defaultsGetAgcLevel();
  public final static native boolean MediaSessionMgr_defaultsSetVadEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetGetVadEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetNoiseSuppEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetNoiseSuppEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetNoiseSuppLevel(int jarg1);
  public final static native int MediaSessionMgr_defaultsGetNoiseSuppLevel();
  public final static native boolean MediaSessionMgr_defaultsSet100relEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGet100relEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetScreenSize(int jarg1, int jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetAudioGain(int jarg1, int jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetAudioPtime(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetAudioChannels(int jarg1, int jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetRtpPortRange(int jarg1, int jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetRtpSymetricEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetMediaType(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetVolume(int jarg1);
  public final static native int MediaSessionMgr_defaultsGetVolume();
  public final static native boolean MediaSessionMgr_defaultsSetInviteSessionTimers(int jarg1, String jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetSRtpMode(int jarg1);
  public final static native int MediaSessionMgr_defaultsGetSRtpMode();
  public final static native boolean MediaSessionMgr_defaultsSetSRtpType(int jarg1);
  public final static native int MediaSessionMgr_defaultsGetSRtpType();
  public final static native boolean MediaSessionMgr_defaultsSetRtcpEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetRtcpEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetRtcpMuxEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetRtcpMuxEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetStunEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetIceStunEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetIceTurnEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetStunServer(String jarg1, int jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetStunCred(String jarg1, String jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetIceEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetByPassEncoding(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetByPassEncoding();
  public final static native boolean MediaSessionMgr_defaultsSetByPassDecoding(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetByPassDecoding();
  public final static native boolean MediaSessionMgr_defaultsSetVideoJbEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetVideoJbEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetVideoZeroArtifactsEnabled(boolean jarg1);
  public final static native boolean MediaSessionMgr_defaultsGetVideoZeroArtifactsEnabled();
  public final static native boolean MediaSessionMgr_defaultsSetRtpBuffSize(long jarg1);
  public final static native long MediaSessionMgr_defaultsGetRtpBuffSize();
  public final static native boolean MediaSessionMgr_defaultsSetAvpfTail(long jarg1, long jarg2);
  public final static native boolean MediaSessionMgr_defaultsSetAvpfMode(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetOpusMaxCaptureRate(long jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetOpusMaxPlaybackRate(long jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetMaxFds(int jarg1);
  public final static native boolean MediaSessionMgr_defaultsSetPoCQoE(int jarg1, int jarg2);
  public final static native void delete_MediaContent(long jarg1);
  public final static native String MediaContent_getType(long jarg1, MediaContent jarg1_);
  public final static native long MediaContent_getDataLength(long jarg1, MediaContent jarg1_);
  public final static native long MediaContent_getData(long jarg1, MediaContent jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long MediaContent_parse__SWIG_0(java.nio.ByteBuffer jarg1, long jarg2, String jarg3);
  public final static native long MediaContent_parse__SWIG_1(java.nio.ByteBuffer jarg1, long jarg2);
  public final static native long MediaContent_getPayloadLength(long jarg1, MediaContent jarg1_);
  public final static native long MediaContent_getPayload(long jarg1, MediaContent jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native void delete_MediaContentCPIM(long jarg1);
  public final static native long MediaContentCPIM_getPayloadLength(long jarg1, MediaContentCPIM jarg1_);
  public final static native long MediaContentCPIM_getPayload(long jarg1, MediaContentCPIM jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native String MediaContentCPIM_getHeaderValue(long jarg1, MediaContentCPIM jarg1_, String jarg2);
  public final static native long new_SipUri__SWIG_0(String jarg1, String jarg2);
  public final static native long new_SipUri__SWIG_1(String jarg1);
  public final static native void delete_SipUri(long jarg1);
  public final static native boolean SipUri_isValid__SWIG_0(String jarg1);
  public final static native boolean SipUri_isValid__SWIG_1(long jarg1, SipUri jarg1_);
  public final static native String SipUri_getScheme(long jarg1, SipUri jarg1_);
  public final static native String SipUri_getHost(long jarg1, SipUri jarg1_);
  public final static native int SipUri_getPort(long jarg1, SipUri jarg1_);
  public final static native String SipUri_getUserName(long jarg1, SipUri jarg1_);
  public final static native String SipUri_getPassword(long jarg1, SipUri jarg1_);
  public final static native String SipUri_getDisplayName(long jarg1, SipUri jarg1_);
  public final static native String SipUri_getParamValue(long jarg1, SipUri jarg1_, String jarg2);
  public final static native void SipUri_setDisplayName(long jarg1, SipUri jarg1_, String jarg2);
  public final static native long new_SdpMessage();
  public final static native void delete_SdpMessage(long jarg1);
  public final static native String SdpMessage_getSdpHeaderValue__SWIG_0(long jarg1, SdpMessage jarg1_, String jarg2, char jarg3, long jarg4);
  public final static native String SdpMessage_getSdpHeaderValue__SWIG_1(long jarg1, SdpMessage jarg1_, String jarg2, char jarg3);
  public final static native String SdpMessage_getSdpHeaderAValue(long jarg1, SdpMessage jarg1_, String jarg2, String jarg3);
  public final static native int SdpMessage_getSdpHeaderMPort__SWIG_0(long jarg1, SdpMessage jarg1_, String jarg2, long jarg3);
  public final static native int SdpMessage_getSdpHeaderMPort__SWIG_1(long jarg1, SdpMessage jarg1_, String jarg2);
  public final static native String SdpMessage_getSdpHeaderCAddr(long jarg1, SdpMessage jarg1_, String jarg2, long jarg3);
  public final static native String SdpMessage_getSdpHeaderCAddrType(long jarg1, SdpMessage jarg1_, String jarg2, long jarg3);
  public final static native long new_SipMessage();
  public final static native void delete_SipMessage(long jarg1);
  public final static native boolean SipMessage_isResponse(long jarg1, SipMessage jarg1_);
  public final static native int SipMessage_getRequestType(long jarg1, SipMessage jarg1_);
  public final static native short SipMessage_getResponseCode(long jarg1, SipMessage jarg1_);
  public final static native String SipMessage_getResponsePhrase(long jarg1, SipMessage jarg1_);
  public final static native String SipMessage_getSipHeaderValue__SWIG_0(long jarg1, SipMessage jarg1_, String jarg2, long jarg3);
  public final static native String SipMessage_getSipHeaderValue__SWIG_1(long jarg1, SipMessage jarg1_, String jarg2);
  public final static native String SipMessage_getSipHeaderParamValue__SWIG_0(long jarg1, SipMessage jarg1_, String jarg2, String jarg3, long jarg4);
  public final static native String SipMessage_getSipHeaderParamValue__SWIG_1(long jarg1, SipMessage jarg1_, String jarg2, String jarg3);
  public final static native long SipMessage_getSipContentLength(long jarg1, SipMessage jarg1_);
  public final static native long SipMessage_getSipContentSdp(long jarg1, SipMessage jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long SipMessage_getSipContentMbms(long jarg1, SipMessage jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long SipMessage_getSipContentContetType(long jarg1, SipMessage jarg1_, java.nio.ByteBuffer jarg2, long jarg3, String jarg4);
  public final static native long SipMessage_getSipContent(long jarg1, SipMessage jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long SipMessage_getSdpMessage(long jarg1, SipMessage jarg1_);
  public final static native void delete_SipEvent(long jarg1);
  public final static native short SipEvent_getCode(long jarg1, SipEvent jarg1_);
  public final static native String SipEvent_getPhrase(long jarg1, SipEvent jarg1_);
  public final static native long SipEvent_getBaseSession(long jarg1, SipEvent jarg1_);
  public final static native long SipEvent_getSipMessage(long jarg1, SipEvent jarg1_);
  public final static native void delete_DialogEvent(long jarg1);
  public final static native void delete_StackEvent(long jarg1);
  public final static native void delete_InviteEvent(long jarg1);
  public final static native int InviteEvent_getType(long jarg1, InviteEvent jarg1_);
  public final static native int InviteEvent_getMediaType(long jarg1, InviteEvent jarg1_);
  public final static native long InviteEvent_getSession(long jarg1, InviteEvent jarg1_);
  public final static native long InviteEvent_takeCallSessionOwnership(long jarg1, InviteEvent jarg1_);
  public final static native long InviteEvent_takeMsrpSessionOwnership(long jarg1, InviteEvent jarg1_);
  public final static native void delete_MessagingEvent(long jarg1);
  public final static native int MessagingEvent_getType(long jarg1, MessagingEvent jarg1_);
  public final static native long MessagingEvent_getSession(long jarg1, MessagingEvent jarg1_);
  public final static native long MessagingEvent_takeSessionOwnership(long jarg1, MessagingEvent jarg1_);
  public final static native void delete_MessagingLocationEvent(long jarg1);
  public final static native int MessagingLocationEvent_getType(long jarg1, MessagingLocationEvent jarg1_);
  public final static native long MessagingLocationEvent_getSession(long jarg1, MessagingLocationEvent jarg1_);
  public final static native long MessagingLocationEvent_takeSessionOwnership(long jarg1, MessagingLocationEvent jarg1_);
  public final static native void delete_MessagingAffiliationEvent(long jarg1);
  public final static native int MessagingAffiliationEvent_getType(long jarg1, MessagingAffiliationEvent jarg1_);
  public final static native long MessagingAffiliationEvent_getSession(long jarg1, MessagingAffiliationEvent jarg1_);
  public final static native long MessagingAffiliationEvent_takeSessionOwnership(long jarg1, MessagingAffiliationEvent jarg1_);
  public final static native void delete_MessagingMbmsEvent(long jarg1);
  public final static native int MessagingMbmsEvent_getType(long jarg1, MessagingMbmsEvent jarg1_);
  public final static native long MessagingMbmsEvent_getSession(long jarg1, MessagingMbmsEvent jarg1_);
  public final static native long MessagingMbmsEvent_takeSessionOwnership(long jarg1, MessagingMbmsEvent jarg1_);
  public final static native void delete_InfoEvent(long jarg1);
  public final static native int InfoEvent_getType(long jarg1, InfoEvent jarg1_);
  public final static native long InfoEvent_getSession(long jarg1, InfoEvent jarg1_);
  public final static native long InfoEvent_takeSessionOwnership(long jarg1, InfoEvent jarg1_);
  public final static native void delete_OptionsEvent(long jarg1);
  public final static native int OptionsEvent_getType(long jarg1, OptionsEvent jarg1_);
  public final static native long OptionsEvent_getSession(long jarg1, OptionsEvent jarg1_);
  public final static native long OptionsEvent_takeSessionOwnership(long jarg1, OptionsEvent jarg1_);
  public final static native void delete_PublicationEvent(long jarg1);
  public final static native int PublicationEvent_getType(long jarg1, PublicationEvent jarg1_);
  public final static native long PublicationEvent_getSession(long jarg1, PublicationEvent jarg1_);
  public final static native long PublicationEvent_takeSessionOwnership(long jarg1, PublicationEvent jarg1_);
  public final static native void delete_PublicationAffiliationEvent(long jarg1);
  public final static native int PublicationAffiliationEvent_getType(long jarg1, PublicationAffiliationEvent jarg1_);
  public final static native long PublicationAffiliationEvent_getSession(long jarg1, PublicationAffiliationEvent jarg1_);
  public final static native long PublicationAffiliationEvent_takeSessionOwnership(long jarg1, PublicationAffiliationEvent jarg1_);
  public final static native void delete_PublicationAuthenticationEvent(long jarg1);
  public final static native int PublicationAuthenticationEvent_getType(long jarg1, PublicationAuthenticationEvent jarg1_);
  public final static native long PublicationAuthenticationEvent_getSession(long jarg1, PublicationAuthenticationEvent jarg1_);
  public final static native long PublicationAuthenticationEvent_takeSessionOwnership(long jarg1, PublicationAuthenticationEvent jarg1_);
  public final static native void delete_RegistrationAuthenticationEvent(long jarg1);
  public final static native int RegistrationAuthenticationEvent_getType(long jarg1, RegistrationAuthenticationEvent jarg1_);
  public final static native void delete_RegistrationEvent(long jarg1);
  public final static native int RegistrationEvent_getType(long jarg1, RegistrationEvent jarg1_);
  public final static native long RegistrationEvent_getSession(long jarg1, RegistrationEvent jarg1_);
  public final static native long RegistrationEvent_takeSessionOwnership(long jarg1, RegistrationEvent jarg1_);
  public final static native void delete_SubscriptionEvent(long jarg1);
  public final static native int SubscriptionEvent_getType(long jarg1, SubscriptionEvent jarg1_);
  public final static native long SubscriptionEvent_getSession(long jarg1, SubscriptionEvent jarg1_);
  public final static native long SubscriptionEvent_takeSessionOwnership(long jarg1, SubscriptionEvent jarg1_);
  public final static native void delete_SubscriptionAffiliationEvent(long jarg1);
  public final static native int SubscriptionAffiliationEvent_getType(long jarg1, SubscriptionAffiliationEvent jarg1_);
  public final static native long SubscriptionAffiliationEvent_getSession(long jarg1, SubscriptionAffiliationEvent jarg1_);
  public final static native long SubscriptionAffiliationEvent_takeSessionOwnership(long jarg1, SubscriptionAffiliationEvent jarg1_);
  public final static native void delete_SubscriptionCMSEvent(long jarg1);
  public final static native int SubscriptionCMSEvent_getType(long jarg1, SubscriptionCMSEvent jarg1_);
  public final static native long SubscriptionCMSEvent_getSession(long jarg1, SubscriptionCMSEvent jarg1_);
  public final static native long SubscriptionCMSEvent_takeSessionOwnership(long jarg1, SubscriptionCMSEvent jarg1_);
  public final static native void delete_SubscriptionGMSEvent(long jarg1);
  public final static native int SubscriptionGMSEvent_getType(long jarg1, SubscriptionGMSEvent jarg1_);
  public final static native long SubscriptionGMSEvent_getSession(long jarg1, SubscriptionGMSEvent jarg1_);
  public final static native long SubscriptionGMSEvent_takeSessionOwnership(long jarg1, SubscriptionGMSEvent jarg1_);
  public final static native void delete_T140CallbackData(long jarg1);
  public final static native int T140CallbackData_getType(long jarg1, T140CallbackData jarg1_);
  public final static native long T140CallbackData_getSize(long jarg1, T140CallbackData jarg1_);
  public final static native long T140CallbackData_getData(long jarg1, T140CallbackData jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long new_T140Callback();
  public final static native void delete_T140Callback(long jarg1);
  public final static native int T140Callback_ondata(long jarg1, T140Callback jarg1_, long jarg2, T140CallbackData jarg2_);
  public final static native int T140Callback_ondataSwigExplicitT140Callback(long jarg1, T140Callback jarg1_, long jarg2, T140CallbackData jarg2_);
  public final static native void T140Callback_director_connect(T140Callback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void T140Callback_change_ownership(T140Callback obj, long cptr, boolean take_or_release);
  public final static native long new_SipSession(long jarg1, SipStack jarg1_);
  public final static native void delete_SipSession(long jarg1);
  public final static native boolean SipSession_haveOwnership(long jarg1, SipSession jarg1_);
  public final static native boolean SipSession_startMBMSManager(long jarg1, SipSession jarg1_, boolean jarg2);
  public final static native boolean SipSession_addHeader(long jarg1, SipSession jarg1_, String jarg2, String jarg3);
  public final static native boolean SipSession_removeHeader(long jarg1, SipSession jarg1_, String jarg2);
  public final static native boolean SipSession_addCaps__SWIG_0(long jarg1, SipSession jarg1_, String jarg2, String jarg3);
  public final static native boolean SipSession_addCaps__SWIG_1(long jarg1, SipSession jarg1_, String jarg2);
  public final static native boolean SipSession_removeCaps(long jarg1, SipSession jarg1_, String jarg2);
  public final static native boolean SipSession_setExpires(long jarg1, SipSession jarg1_, long jarg2);
  public final static native boolean SipSession_setFromUri__SWIG_0(long jarg1, SipSession jarg1_, String jarg2);
  public final static native boolean SipSession_setFromUri__SWIG_1(long jarg1, SipSession jarg1_, long jarg2, SipUri jarg2_);
  public final static native boolean SipSession_setToUri__SWIG_0(long jarg1, SipSession jarg1_, String jarg2);
  public final static native boolean SipSession_setToUri__SWIG_1(long jarg1, SipSession jarg1_, long jarg2, SipUri jarg2_);
  public final static native boolean SipSession_setSilentHangup(long jarg1, SipSession jarg1_, boolean jarg2);
  public final static native boolean SipSession_addSigCompCompartment(long jarg1, SipSession jarg1_, String jarg2);
  public final static native boolean SipSession_removeSigCompCompartment(long jarg1, SipSession jarg1_);
  public final static native long SipSession_getId(long jarg1, SipSession jarg1_);
  public final static native long new_InviteSession(long jarg1, SipStack jarg1_);
  public final static native void delete_InviteSession(long jarg1);
  public final static native boolean InviteSession_accept__SWIG_0(long jarg1, InviteSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean InviteSession_accept__SWIG_1(long jarg1, InviteSession jarg1_);
  public final static native boolean InviteSession_hangup__SWIG_0(long jarg1, InviteSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean InviteSession_hangup__SWIG_1(long jarg1, InviteSession jarg1_);
  public final static native boolean InviteSession_reject__SWIG_0(long jarg1, InviteSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean InviteSession_reject__SWIG_1(long jarg1, InviteSession jarg1_);
  public final static native boolean InviteSession_sendInfo__SWIG_0(long jarg1, InviteSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean InviteSession_sendInfo__SWIG_1(long jarg1, InviteSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long InviteSession_getMediaMgr(long jarg1, InviteSession jarg1_);
  public final static native long new_CallSession(long jarg1, SipStack jarg1_);
  public final static native void delete_CallSession(long jarg1);
  public final static native boolean CallSession_callAudio__SWIG_0(long jarg1, CallSession jarg1_, String jarg2, long jarg3, ActionConfig jarg3_);
  public final static native boolean CallSession_callAudio__SWIG_1(long jarg1, CallSession jarg1_, String jarg2);
  public final static native boolean CallSession_callAudio__SWIG_2(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_, long jarg3, ActionConfig jarg3_);
  public final static native boolean CallSession_callAudio__SWIG_3(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_);
  public final static native boolean CallSession_callAudioVideo__SWIG_0(long jarg1, CallSession jarg1_, String jarg2, long jarg3, ActionConfig jarg3_);
  public final static native boolean CallSession_callAudioVideo__SWIG_1(long jarg1, CallSession jarg1_, String jarg2);
  public final static native boolean CallSession_callAudioVideo__SWIG_2(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_, long jarg3, ActionConfig jarg3_);
  public final static native boolean CallSession_callAudioVideo__SWIG_3(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_);
  public final static native boolean CallSession_callVideo__SWIG_0(long jarg1, CallSession jarg1_, String jarg2, long jarg3, ActionConfig jarg3_);
  public final static native boolean CallSession_callVideo__SWIG_1(long jarg1, CallSession jarg1_, String jarg2);
  public final static native boolean CallSession_callVideo__SWIG_2(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_, long jarg3, ActionConfig jarg3_);
  public final static native boolean CallSession_callVideo__SWIG_3(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_);
  public final static native boolean CallSession_call__SWIG_0(long jarg1, CallSession jarg1_, String jarg2, int jarg3, boolean jarg4, long jarg5, ActionConfig jarg5_);
  public final static native boolean CallSession_call__SWIG_1(long jarg1, CallSession jarg1_, String jarg2, int jarg3, boolean jarg4);
  public final static native boolean CallSession_call__SWIG_2(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_, int jarg3, boolean jarg4, long jarg5, ActionConfig jarg5_);
  public final static native boolean CallSession_call__SWIG_3(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_, int jarg3, boolean jarg4);
  public final static native boolean CallSession_callEmergency__SWIG_0(long jarg1, CallSession jarg1_, String jarg2, int jarg3, boolean jarg4, String jarg5, int jarg6, long jarg7, ActionConfig jarg7_);
  public final static native boolean CallSession_callEmergency__SWIG_1(long jarg1, CallSession jarg1_, String jarg2, int jarg3, boolean jarg4, String jarg5, int jarg6);
  public final static native boolean CallSession_callEmergency__SWIG_2(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_, int jarg3, boolean jarg4, String jarg5, int jarg6, long jarg7, ActionConfig jarg7_);
  public final static native boolean CallSession_callEmergency__SWIG_3(long jarg1, CallSession jarg1_, long jarg2, SipUri jarg2_, int jarg3, boolean jarg4, String jarg5, int jarg6);
  public final static native boolean CallSession_callGroup(long jarg1, CallSession jarg1_, long jarg2, int jarg3, int jarg4, long jarg5, ActionConfig jarg5_);
  public final static native boolean CallSession_setSessionTimer(long jarg1, CallSession jarg1_, long jarg2, String jarg3);
  public final static native boolean CallSession_set100rel(long jarg1, CallSession jarg1_, boolean jarg2);
  public final static native boolean CallSession_setRtcp(long jarg1, CallSession jarg1_, boolean jarg2);
  public final static native boolean CallSession_setRtcpMux(long jarg1, CallSession jarg1_, boolean jarg2);
  public final static native boolean CallSession_setSRtpMode(long jarg1, CallSession jarg1_, int jarg2);
  public final static native boolean CallSession_setAvpfMode(long jarg1, CallSession jarg1_, int jarg2);
  public final static native boolean CallSession_setICE(long jarg1, CallSession jarg1_, boolean jarg2);
  public final static native boolean CallSession_setICEStun(long jarg1, CallSession jarg1_, boolean jarg2);
  public final static native boolean CallSession_setICETurn(long jarg1, CallSession jarg1_, boolean jarg2);
  public final static native boolean CallSession_setSTUNServer(long jarg1, CallSession jarg1_, String jarg2, int jarg3);
  public final static native boolean CallSession_setSTUNCred(long jarg1, CallSession jarg1_, String jarg2, String jarg3);
  public final static native boolean CallSession_setVideoFps(long jarg1, CallSession jarg1_, int jarg2);
  public final static native boolean CallSession_setVideoBandwidthUploadMax(long jarg1, CallSession jarg1_, int jarg2);
  public final static native boolean CallSession_setVideoBandwidthDownloadMax(long jarg1, CallSession jarg1_, int jarg2);
  public final static native boolean CallSession_setVideoPrefSize(long jarg1, CallSession jarg1_, int jarg2);
  public final static native boolean CallSession_setQoS(long jarg1, CallSession jarg1_, int jarg2, int jarg3);
  public final static native boolean CallSession_setPoCQoE(long jarg1, CallSession jarg1_, int jarg2, int jarg3);
  public final static native boolean CallSession_hold__SWIG_0(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean CallSession_hold__SWIG_1(long jarg1, CallSession jarg1_);
  public final static native boolean CallSession_resume__SWIG_0(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean CallSession_resume__SWIG_1(long jarg1, CallSession jarg1_);
  public final static native boolean CallSession_transfer__SWIG_0(long jarg1, CallSession jarg1_, String jarg2, long jarg3, ActionConfig jarg3_);
  public final static native boolean CallSession_transfer__SWIG_1(long jarg1, CallSession jarg1_, String jarg2);
  public final static native boolean CallSession_acceptTransfer__SWIG_0(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean CallSession_acceptTransfer__SWIG_1(long jarg1, CallSession jarg1_);
  public final static native boolean CallSession_rejectTransfer__SWIG_0(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean CallSession_rejectTransfer__SWIG_1(long jarg1, CallSession jarg1_);
  public final static native boolean CallSession_sendDTMF(long jarg1, CallSession jarg1_, int jarg2);
  public final static native long CallSession_getSessionTransferId(long jarg1, CallSession jarg1_);
  public final static native boolean CallSession_sendT140Data__SWIG_0(long jarg1, CallSession jarg1_, int jarg2, java.nio.ByteBuffer jarg3, long jarg4);
  public final static native boolean CallSession_sendT140Data__SWIG_1(long jarg1, CallSession jarg1_, int jarg2, java.nio.ByteBuffer jarg3);
  public final static native boolean CallSession_sendT140Data__SWIG_2(long jarg1, CallSession jarg1_, int jarg2);
  public final static native boolean CallSession_setT140Callback(long jarg1, CallSession jarg1_, long jarg2, T140Callback jarg2_);
  public final static native boolean CallSession_setMcpttCallback(long jarg1, CallSession jarg1_, long jarg2, McpttCallback jarg2_);
  public final static native boolean CallSession_setMcpttMbmsCallback(long jarg1, CallSession jarg1_, long jarg2, McpttMbmsCallback jarg2_);
  public final static native boolean CallSession_requestMcpttToken__SWIG_0(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean CallSession_requestMcpttToken__SWIG_1(long jarg1, CallSession jarg1_);
  public final static native boolean CallSession_releaseMcpttToken__SWIG_0(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean CallSession_releaseMcpttToken__SWIG_1(long jarg1, CallSession jarg1_);
  public final static native boolean CallSession_startMbmsManager(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_, String jarg3, int jarg4, String jarg5, int jarg6);
  public final static native boolean CallSession_stopMbmsManager(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean CallSession_startMbmsMedia(long jarg1, CallSession jarg1_, long jarg2, ActionConfig jarg2_, String jarg3, int jarg4, int jarg5);
  public final static native String CallSession_getPTTMcpttGroupIdentity(long jarg1, CallSession jarg1_);
  public final static native int CallSession_getPTTMcpttGroupMembers(long jarg1, CallSession jarg1_);
  public final static native String CallSession_getPTTMcpttGroupMemberAtPosition(long jarg1, CallSession jarg1_, int jarg2);
  public final static native String CallSession_getPttMcpttEmergencyResourcePriorityString(long jarg1, CallSession jarg1_);
  public final static native int CallSession_getPttMcpttEmergencyResourcePriority(long jarg1, CallSession jarg1_);
  public final static native String CallSession_getSipPartyUri(long jarg1, CallSession jarg1_);
  public final static native long new_MsrpSession(long jarg1, SipStack jarg1_, long jarg2, MsrpCallback jarg2_);
  public final static native void delete_MsrpSession(long jarg1);
  public final static native boolean MsrpSession_setCallback(long jarg1, MsrpSession jarg1_, long jarg2, MsrpCallback jarg2_);
  public final static native boolean MsrpSession_callMsrp__SWIG_0(long jarg1, MsrpSession jarg1_, String jarg2, long jarg3, ActionConfig jarg3_);
  public final static native boolean MsrpSession_callMsrp__SWIG_1(long jarg1, MsrpSession jarg1_, String jarg2);
  public final static native boolean MsrpSession_callMsrp__SWIG_2(long jarg1, MsrpSession jarg1_, long jarg2, SipUri jarg2_, long jarg3, ActionConfig jarg3_);
  public final static native boolean MsrpSession_callMsrp__SWIG_3(long jarg1, MsrpSession jarg1_, long jarg2, SipUri jarg2_);
  public final static native boolean MsrpSession_sendMessage__SWIG_0(long jarg1, MsrpSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean MsrpSession_sendMessage__SWIG_1(long jarg1, MsrpSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean MsrpSession_sendFile__SWIG_0(long jarg1, MsrpSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MsrpSession_sendFile__SWIG_1(long jarg1, MsrpSession jarg1_);
  public final static native long new_McpttSession(long jarg1, SipStack jarg1_, long jarg2, McpttCallback jarg2_);
  public final static native void delete_McpttSession(long jarg1);
  public final static native boolean McpttSession_setCallback(long jarg1, McpttSession jarg1_, long jarg2, McpttCallback jarg2_);
  public final static native boolean McpttSession_requestToken__SWIG_0(long jarg1, McpttSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean McpttSession_requestToken__SWIG_1(long jarg1, McpttSession jarg1_);
  public final static native long new_MessagingSession(long jarg1, SipStack jarg1_);
  public final static native void delete_MessagingSession(long jarg1);
  public final static native boolean MessagingSession_send__SWIG_0(long jarg1, MessagingSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean MessagingSession_send__SWIG_1(long jarg1, MessagingSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean MessagingSession_accept__SWIG_0(long jarg1, MessagingSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingSession_accept__SWIG_1(long jarg1, MessagingSession jarg1_);
  public final static native boolean MessagingSession_reject__SWIG_0(long jarg1, MessagingSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingSession_reject__SWIG_1(long jarg1, MessagingSession jarg1_);
  public final static native long new_MessagingLocationSession(long jarg1, SipStack jarg1_);
  public final static native void delete_MessagingLocationSession(long jarg1);
  public final static native boolean MessagingLocationSession_send__SWIG_0(long jarg1, MessagingLocationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean MessagingLocationSession_send__SWIG_1(long jarg1, MessagingLocationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean MessagingLocationSession_accept__SWIG_0(long jarg1, MessagingLocationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingLocationSession_accept__SWIG_1(long jarg1, MessagingLocationSession jarg1_);
  public final static native boolean MessagingLocationSession_reject__SWIG_0(long jarg1, MessagingLocationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingLocationSession_reject__SWIG_1(long jarg1, MessagingLocationSession jarg1_);
  public final static native long new_MessagingAffiliationSession(long jarg1, SipStack jarg1_);
  public final static native void delete_MessagingAffiliationSession(long jarg1);
  public final static native boolean MessagingAffiliationSession_send__SWIG_0(long jarg1, MessagingAffiliationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean MessagingAffiliationSession_send__SWIG_1(long jarg1, MessagingAffiliationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean MessagingAffiliationSession_accept__SWIG_0(long jarg1, MessagingAffiliationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingAffiliationSession_accept__SWIG_1(long jarg1, MessagingAffiliationSession jarg1_);
  public final static native boolean MessagingAffiliationSession_reject__SWIG_0(long jarg1, MessagingAffiliationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingAffiliationSession_reject__SWIG_1(long jarg1, MessagingAffiliationSession jarg1_);
  public final static native long new_MessagingMbmsSession(long jarg1, SipStack jarg1_);
  public final static native void delete_MessagingMbmsSession(long jarg1);
  public final static native boolean MessagingMbmsSession_send__SWIG_0(long jarg1, MessagingMbmsSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean MessagingMbmsSession_send__SWIG_1(long jarg1, MessagingMbmsSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean MessagingMbmsSession_accept__SWIG_0(long jarg1, MessagingMbmsSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingMbmsSession_accept__SWIG_1(long jarg1, MessagingMbmsSession jarg1_);
  public final static native boolean MessagingMbmsSession_reject__SWIG_0(long jarg1, MessagingMbmsSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean MessagingMbmsSession_reject__SWIG_1(long jarg1, MessagingMbmsSession jarg1_);
  public final static native long new_InfoSession(long jarg1, SipStack jarg1_);
  public final static native void delete_InfoSession(long jarg1);
  public final static native boolean InfoSession_send__SWIG_0(long jarg1, InfoSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean InfoSession_send__SWIG_1(long jarg1, InfoSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean InfoSession_accept__SWIG_0(long jarg1, InfoSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean InfoSession_accept__SWIG_1(long jarg1, InfoSession jarg1_);
  public final static native boolean InfoSession_reject__SWIG_0(long jarg1, InfoSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean InfoSession_reject__SWIG_1(long jarg1, InfoSession jarg1_);
  public final static native long new_OptionsSession(long jarg1, SipStack jarg1_);
  public final static native void delete_OptionsSession(long jarg1);
  public final static native boolean OptionsSession_send__SWIG_0(long jarg1, OptionsSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean OptionsSession_send__SWIG_1(long jarg1, OptionsSession jarg1_);
  public final static native boolean OptionsSession_accept__SWIG_0(long jarg1, OptionsSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean OptionsSession_accept__SWIG_1(long jarg1, OptionsSession jarg1_);
  public final static native boolean OptionsSession_reject__SWIG_0(long jarg1, OptionsSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean OptionsSession_reject__SWIG_1(long jarg1, OptionsSession jarg1_);
  public final static native long new_PublicationSession(long jarg1, SipStack jarg1_);
  public final static native void delete_PublicationSession(long jarg1);
  public final static native boolean PublicationSession_publish__SWIG_0(long jarg1, PublicationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean PublicationSession_publish__SWIG_1(long jarg1, PublicationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean PublicationSession_unPublish__SWIG_0(long jarg1, PublicationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean PublicationSession_unPublish__SWIG_1(long jarg1, PublicationSession jarg1_);
  public final static native long new_PublicationAffiliationSession(long jarg1, SipStack jarg1_);
  public final static native void delete_PublicationAffiliationSession(long jarg1);
  public final static native boolean PublicationAffiliationSession_publish__SWIG_0(long jarg1, PublicationAffiliationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean PublicationAffiliationSession_publish__SWIG_1(long jarg1, PublicationAffiliationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean PublicationAffiliationSession_unPublish__SWIG_0(long jarg1, PublicationAffiliationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean PublicationAffiliationSession_unPublish__SWIG_1(long jarg1, PublicationAffiliationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long new_PublicationAuthenticationSession(long jarg1, SipStack jarg1_);
  public final static native void delete_PublicationAuthenticationSession(long jarg1);
  public final static native boolean PublicationAuthenticationSession_publish__SWIG_0(long jarg1, PublicationAuthenticationSession jarg1_, String jarg2, String jarg3, java.nio.ByteBuffer jarg4, long jarg5, long jarg6, ActionConfig jarg6_);
  public final static native boolean PublicationAuthenticationSession_publish__SWIG_1(long jarg1, PublicationAuthenticationSession jarg1_, String jarg2, String jarg3, java.nio.ByteBuffer jarg4, long jarg5);
  public final static native boolean PublicationAuthenticationSession_unPublish__SWIG_0(long jarg1, PublicationAuthenticationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean PublicationAuthenticationSession_unPublish__SWIG_1(long jarg1, PublicationAuthenticationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long new_RegistrationSession(long jarg1, SipStack jarg1_);
  public final static native void delete_RegistrationSession(long jarg1);
  public final static native boolean RegistrationSession_register___SWIG_0(long jarg1, RegistrationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean RegistrationSession_register___SWIG_1(long jarg1, RegistrationSession jarg1_);
  public final static native boolean RegistrationSession_registerWithMcpttInfo__SWIG_0(long jarg1, RegistrationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, long jarg4, ActionConfig jarg4_);
  public final static native boolean RegistrationSession_registerWithMcpttInfo__SWIG_1(long jarg1, RegistrationSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean RegistrationSession_unRegister__SWIG_0(long jarg1, RegistrationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean RegistrationSession_unRegister__SWIG_1(long jarg1, RegistrationSession jarg1_);
  public final static native boolean RegistrationSession_accept__SWIG_0(long jarg1, RegistrationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean RegistrationSession_accept__SWIG_1(long jarg1, RegistrationSession jarg1_);
  public final static native boolean RegistrationSession_reject__SWIG_0(long jarg1, RegistrationSession jarg1_, long jarg2, ActionConfig jarg2_);
  public final static native boolean RegistrationSession_reject__SWIG_1(long jarg1, RegistrationSession jarg1_);
  public final static native long new_SubscriptionSession(long jarg1, SipStack jarg1_);
  public final static native void delete_SubscriptionSession(long jarg1);
  public final static native boolean SubscriptionSession_subscribe(long jarg1, SubscriptionSession jarg1_);
  public final static native boolean SubscriptionSession_unSubscribe(long jarg1, SubscriptionSession jarg1_);
  public final static native long new_SubscriptionAffiliationSession(long jarg1, SipStack jarg1_);
  public final static native void delete_SubscriptionAffiliationSession(long jarg1);
  public final static native boolean SubscriptionAffiliationSession_subscribeAffiliation(long jarg1, SubscriptionAffiliationSession jarg1_);
  public final static native boolean SubscriptionAffiliationSession_unSubscribeAffiliation(long jarg1, SubscriptionAffiliationSession jarg1_);
  public final static native long new_SubscriptionCMSSession(long jarg1, SipStack jarg1_);
  public final static native void delete_SubscriptionCMSSession(long jarg1);
  public final static native boolean SubscriptionCMSSession_subscribeCMS__SWIG_0(long jarg1, SubscriptionCMSSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, java.nio.ByteBuffer jarg4, long jarg5, long jarg6, ActionConfig jarg6_);
  public final static native boolean SubscriptionCMSSession_subscribeCMS__SWIG_1(long jarg1, SubscriptionCMSSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, java.nio.ByteBuffer jarg4, long jarg5);
  public final static native boolean SubscriptionCMSSession_unSubscribeCMS(long jarg1, SubscriptionCMSSession jarg1_);
  public final static native long new_SubscriptionGMSSession(long jarg1, SipStack jarg1_);
  public final static native void delete_SubscriptionGMSSession(long jarg1);
  public final static native boolean SubscriptionGMSSession_subscribeGMS__SWIG_0(long jarg1, SubscriptionGMSSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, java.nio.ByteBuffer jarg4, long jarg5, long jarg6, ActionConfig jarg6_);
  public final static native boolean SubscriptionGMSSession_subscribeGMS__SWIG_1(long jarg1, SubscriptionGMSSession jarg1_, java.nio.ByteBuffer jarg2, long jarg3, java.nio.ByteBuffer jarg4, long jarg5);
  public final static native boolean SubscriptionGMSSession_unSubscribeGMS(long jarg1, SubscriptionGMSSession jarg1_);
  public final static native void delete_ProxyPluginMgr(long jarg1);
  public final static native long ProxyPluginMgr_createInstance(long jarg1, ProxyPluginMgrCallback jarg1_);
  public final static native long ProxyPluginMgr_getInstance();
  public final static native long ProxyPluginMgr_findPlugin(long jarg1, ProxyPluginMgr jarg1_, java.math.BigInteger jarg2);
  public final static native long ProxyPluginMgr_findAudioConsumer(long jarg1, ProxyPluginMgr jarg1_, java.math.BigInteger jarg2);
  public final static native long ProxyPluginMgr_findVideoConsumer(long jarg1, ProxyPluginMgr jarg1_, java.math.BigInteger jarg2);
  public final static native long ProxyPluginMgr_findAudioProducer(long jarg1, ProxyPluginMgr jarg1_, java.math.BigInteger jarg2);
  public final static native long ProxyPluginMgr_findVideoProducer(long jarg1, ProxyPluginMgr jarg1_, java.math.BigInteger jarg2);
  public final static native long new_ProxyPluginMgrCallback();
  public final static native void delete_ProxyPluginMgrCallback(long jarg1);
  public final static native int ProxyPluginMgrCallback_OnPluginCreated(long jarg1, ProxyPluginMgrCallback jarg1_, java.math.BigInteger jarg2, int jarg3);
  public final static native int ProxyPluginMgrCallback_OnPluginCreatedSwigExplicitProxyPluginMgrCallback(long jarg1, ProxyPluginMgrCallback jarg1_, java.math.BigInteger jarg2, int jarg3);
  public final static native int ProxyPluginMgrCallback_OnPluginDestroyed(long jarg1, ProxyPluginMgrCallback jarg1_, java.math.BigInteger jarg2, int jarg3);
  public final static native int ProxyPluginMgrCallback_OnPluginDestroyedSwigExplicitProxyPluginMgrCallback(long jarg1, ProxyPluginMgrCallback jarg1_, java.math.BigInteger jarg2, int jarg3);
  public final static native void ProxyPluginMgrCallback_director_connect(ProxyPluginMgrCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void ProxyPluginMgrCallback_change_ownership(ProxyPluginMgrCallback obj, long cptr, boolean take_or_release);
  public final static native void delete_ProxyPlugin(long jarg1);
  public final static native int ProxyPlugin_getType(long jarg1, ProxyPlugin jarg1_);
  public final static native java.math.BigInteger ProxyPlugin_getId(long jarg1, ProxyPlugin jarg1_);
  public final static native long new_ProxyAudioConsumerCallback();
  public final static native void delete_ProxyAudioConsumerCallback(long jarg1);
  public final static native int ProxyAudioConsumerCallback_prepare(long jarg1, ProxyAudioConsumerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyAudioConsumerCallback_prepareSwigExplicitProxyAudioConsumerCallback(long jarg1, ProxyAudioConsumerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyAudioConsumerCallback_start(long jarg1, ProxyAudioConsumerCallback jarg1_);
  public final static native int ProxyAudioConsumerCallback_startSwigExplicitProxyAudioConsumerCallback(long jarg1, ProxyAudioConsumerCallback jarg1_);
  public final static native int ProxyAudioConsumerCallback_pause(long jarg1, ProxyAudioConsumerCallback jarg1_);
  public final static native int ProxyAudioConsumerCallback_pauseSwigExplicitProxyAudioConsumerCallback(long jarg1, ProxyAudioConsumerCallback jarg1_);
  public final static native int ProxyAudioConsumerCallback_stop(long jarg1, ProxyAudioConsumerCallback jarg1_);
  public final static native int ProxyAudioConsumerCallback_stopSwigExplicitProxyAudioConsumerCallback(long jarg1, ProxyAudioConsumerCallback jarg1_);
  public final static native void ProxyAudioConsumerCallback_director_connect(ProxyAudioConsumerCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void ProxyAudioConsumerCallback_change_ownership(ProxyAudioConsumerCallback obj, long cptr, boolean take_or_release);
  public final static native void delete_ProxyAudioConsumer(long jarg1);
  public final static native boolean ProxyAudioConsumer_setActualSndCardPlaybackParams(long jarg1, ProxyAudioConsumer jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native boolean ProxyAudioConsumer_queryForResampler(long jarg1, ProxyAudioConsumer jarg1_, int jarg2, int jarg3, int jarg4, int jarg5, int jarg6);
  public final static native boolean ProxyAudioConsumer_setPullBuffer(long jarg1, ProxyAudioConsumer jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long ProxyAudioConsumer_pull__SWIG_0(long jarg1, ProxyAudioConsumer jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long ProxyAudioConsumer_pull__SWIG_1(long jarg1, ProxyAudioConsumer jarg1_, java.nio.ByteBuffer jarg2);
  public final static native long ProxyAudioConsumer_pull__SWIG_2(long jarg1, ProxyAudioConsumer jarg1_);
  public final static native boolean ProxyAudioConsumer_setGain(long jarg1, ProxyAudioConsumer jarg1_, long jarg2);
  public final static native long ProxyAudioConsumer_getGain(long jarg1, ProxyAudioConsumer jarg1_);
  public final static native boolean ProxyAudioConsumer_reset(long jarg1, ProxyAudioConsumer jarg1_);
  public final static native void ProxyAudioConsumer_setCallback(long jarg1, ProxyAudioConsumer jarg1_, long jarg2, ProxyAudioConsumerCallback jarg2_);
  public final static native java.math.BigInteger ProxyAudioConsumer_getMediaSessionId(long jarg1, ProxyAudioConsumer jarg1_);
  public final static native boolean ProxyAudioConsumer_registerPlugin();
  public final static native long new_ProxyVideoConsumerCallback();
  public final static native void delete_ProxyVideoConsumerCallback(long jarg1);
  public final static native int ProxyVideoConsumerCallback_prepare(long jarg1, ProxyVideoConsumerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyVideoConsumerCallback_prepareSwigExplicitProxyVideoConsumerCallback(long jarg1, ProxyVideoConsumerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyVideoConsumerCallback_consume(long jarg1, ProxyVideoConsumerCallback jarg1_, long jarg2, ProxyVideoFrame jarg2_);
  public final static native int ProxyVideoConsumerCallback_consumeSwigExplicitProxyVideoConsumerCallback(long jarg1, ProxyVideoConsumerCallback jarg1_, long jarg2, ProxyVideoFrame jarg2_);
  public final static native int ProxyVideoConsumerCallback_bufferCopied(long jarg1, ProxyVideoConsumerCallback jarg1_, long jarg2, long jarg3);
  public final static native int ProxyVideoConsumerCallback_bufferCopiedSwigExplicitProxyVideoConsumerCallback(long jarg1, ProxyVideoConsumerCallback jarg1_, long jarg2, long jarg3);
  public final static native int ProxyVideoConsumerCallback_start(long jarg1, ProxyVideoConsumerCallback jarg1_);
  public final static native int ProxyVideoConsumerCallback_startSwigExplicitProxyVideoConsumerCallback(long jarg1, ProxyVideoConsumerCallback jarg1_);
  public final static native int ProxyVideoConsumerCallback_pause(long jarg1, ProxyVideoConsumerCallback jarg1_);
  public final static native int ProxyVideoConsumerCallback_pauseSwigExplicitProxyVideoConsumerCallback(long jarg1, ProxyVideoConsumerCallback jarg1_);
  public final static native int ProxyVideoConsumerCallback_stop(long jarg1, ProxyVideoConsumerCallback jarg1_);
  public final static native int ProxyVideoConsumerCallback_stopSwigExplicitProxyVideoConsumerCallback(long jarg1, ProxyVideoConsumerCallback jarg1_);
  public final static native void ProxyVideoConsumerCallback_director_connect(ProxyVideoConsumerCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void ProxyVideoConsumerCallback_change_ownership(ProxyVideoConsumerCallback obj, long cptr, boolean take_or_release);
  public final static native void delete_ProxyVideoConsumer(long jarg1);
  public final static native boolean ProxyVideoConsumer_setDisplaySize(long jarg1, ProxyVideoConsumer jarg1_, long jarg2, long jarg3);
  public final static native long ProxyVideoConsumer_getDisplayWidth(long jarg1, ProxyVideoConsumer jarg1_);
  public final static native long ProxyVideoConsumer_getDisplayHeight(long jarg1, ProxyVideoConsumer jarg1_);
  public final static native long ProxyVideoConsumer_getDecodedWidth(long jarg1, ProxyVideoConsumer jarg1_);
  public final static native long ProxyVideoConsumer_getDecodedHeight(long jarg1, ProxyVideoConsumer jarg1_);
  public final static native void ProxyVideoConsumer_setCallback(long jarg1, ProxyVideoConsumer jarg1_, long jarg2, ProxyVideoConsumerCallback jarg2_);
  public final static native boolean ProxyVideoConsumer_setAutoResizeDisplay(long jarg1, ProxyVideoConsumer jarg1_, boolean jarg2);
  public final static native boolean ProxyVideoConsumer_getAutoResizeDisplay(long jarg1, ProxyVideoConsumer jarg1_);
  public final static native boolean ProxyVideoConsumer_setConsumeBuffer(long jarg1, ProxyVideoConsumer jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long ProxyVideoConsumer_pull(long jarg1, ProxyVideoConsumer jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native boolean ProxyVideoConsumer_reset(long jarg1, ProxyVideoConsumer jarg1_);
  public final static native java.math.BigInteger ProxyVideoConsumer_getMediaSessionId(long jarg1, ProxyVideoConsumer jarg1_);
  public final static native boolean ProxyVideoConsumer_registerPlugin();
  public final static native void ProxyVideoConsumer_setDefaultChroma(int jarg1);
  public final static native void ProxyVideoConsumer_setDefaultAutoResizeDisplay(boolean jarg1);
  public final static native void delete_ProxyVideoFrame(long jarg1);
  public final static native long ProxyVideoFrame_getSize(long jarg1, ProxyVideoFrame jarg1_);
  public final static native long ProxyVideoFrame_getContent(long jarg1, ProxyVideoFrame jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long ProxyVideoFrame_getFrameWidth(long jarg1, ProxyVideoFrame jarg1_);
  public final static native long ProxyVideoFrame_getFrameHeight(long jarg1, ProxyVideoFrame jarg1_);
  public final static native long new_ProxyAudioProducerCallback();
  public final static native void delete_ProxyAudioProducerCallback(long jarg1);
  public final static native int ProxyAudioProducerCallback_prepare(long jarg1, ProxyAudioProducerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyAudioProducerCallback_prepareSwigExplicitProxyAudioProducerCallback(long jarg1, ProxyAudioProducerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyAudioProducerCallback_start(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native int ProxyAudioProducerCallback_startSwigExplicitProxyAudioProducerCallback(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native int ProxyAudioProducerCallback_pause(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native int ProxyAudioProducerCallback_pauseSwigExplicitProxyAudioProducerCallback(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native int ProxyAudioProducerCallback_stop(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native int ProxyAudioProducerCallback_stopSwigExplicitProxyAudioProducerCallback(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native int ProxyAudioProducerCallback_fillPushBuffer(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native int ProxyAudioProducerCallback_fillPushBufferSwigExplicitProxyAudioProducerCallback(long jarg1, ProxyAudioProducerCallback jarg1_);
  public final static native void ProxyAudioProducerCallback_director_connect(ProxyAudioProducerCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void ProxyAudioProducerCallback_change_ownership(ProxyAudioProducerCallback obj, long cptr, boolean take_or_release);
  public final static native void delete_ProxyAudioProducer(long jarg1);
  public final static native boolean ProxyAudioProducer_setActualSndCardRecordParams(long jarg1, ProxyAudioProducer jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native boolean ProxyAudioProducer_setPushBuffer__SWIG_0(long jarg1, ProxyAudioProducer jarg1_, java.nio.ByteBuffer jarg2, long jarg3, boolean jarg4);
  public final static native boolean ProxyAudioProducer_setPushBuffer__SWIG_1(long jarg1, ProxyAudioProducer jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native int ProxyAudioProducer_push__SWIG_0(long jarg1, ProxyAudioProducer jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native int ProxyAudioProducer_push__SWIG_1(long jarg1, ProxyAudioProducer jarg1_, java.nio.ByteBuffer jarg2);
  public final static native int ProxyAudioProducer_push__SWIG_2(long jarg1, ProxyAudioProducer jarg1_);
  public final static native boolean ProxyAudioProducer_setGain(long jarg1, ProxyAudioProducer jarg1_, long jarg2);
  public final static native long ProxyAudioProducer_getGain(long jarg1, ProxyAudioProducer jarg1_);
  public final static native void ProxyAudioProducer_setCallback(long jarg1, ProxyAudioProducer jarg1_, long jarg2, ProxyAudioProducerCallback jarg2_);
  public final static native java.math.BigInteger ProxyAudioProducer_getMediaSessionId(long jarg1, ProxyAudioProducer jarg1_);
  public final static native boolean ProxyAudioProducer_registerPlugin();
  public final static native long new_ProxyVideoProducerCallback();
  public final static native void delete_ProxyVideoProducerCallback(long jarg1);
  public final static native int ProxyVideoProducerCallback_prepare(long jarg1, ProxyVideoProducerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyVideoProducerCallback_prepareSwigExplicitProxyVideoProducerCallback(long jarg1, ProxyVideoProducerCallback jarg1_, int jarg2, int jarg3, int jarg4);
  public final static native int ProxyVideoProducerCallback_start(long jarg1, ProxyVideoProducerCallback jarg1_);
  public final static native int ProxyVideoProducerCallback_startSwigExplicitProxyVideoProducerCallback(long jarg1, ProxyVideoProducerCallback jarg1_);
  public final static native int ProxyVideoProducerCallback_pause(long jarg1, ProxyVideoProducerCallback jarg1_);
  public final static native int ProxyVideoProducerCallback_pauseSwigExplicitProxyVideoProducerCallback(long jarg1, ProxyVideoProducerCallback jarg1_);
  public final static native int ProxyVideoProducerCallback_stop(long jarg1, ProxyVideoProducerCallback jarg1_);
  public final static native int ProxyVideoProducerCallback_stopSwigExplicitProxyVideoProducerCallback(long jarg1, ProxyVideoProducerCallback jarg1_);
  public final static native void ProxyVideoProducerCallback_director_connect(ProxyVideoProducerCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void ProxyVideoProducerCallback_change_ownership(ProxyVideoProducerCallback obj, long cptr, boolean take_or_release);
  public final static native void delete_ProxyVideoProducer(long jarg1);
  public final static native int ProxyVideoProducer_getRotation(long jarg1, ProxyVideoProducer jarg1_);
  public final static native boolean ProxyVideoProducer_setRotation(long jarg1, ProxyVideoProducer jarg1_, int jarg2);
  public final static native boolean ProxyVideoProducer_getMirror(long jarg1, ProxyVideoProducer jarg1_);
  public final static native boolean ProxyVideoProducer_setMirror(long jarg1, ProxyVideoProducer jarg1_, boolean jarg2);
  public final static native boolean ProxyVideoProducer_setActualCameraOutputSize(long jarg1, ProxyVideoProducer jarg1_, long jarg2, long jarg3);
  public final static native int ProxyVideoProducer_push(long jarg1, ProxyVideoProducer jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native void ProxyVideoProducer_setCallback(long jarg1, ProxyVideoProducer jarg1_, long jarg2, ProxyVideoProducerCallback jarg2_);
  public final static native java.math.BigInteger ProxyVideoProducer_getMediaSessionId(long jarg1, ProxyVideoProducer jarg1_);
  public final static native boolean ProxyVideoProducer_registerPlugin();
  public final static native void ProxyVideoProducer_setDefaultChroma(int jarg1);
  public final static native long new_SipCallback();
  public final static native void delete_SipCallback(long jarg1);
  public final static native int SipCallback_OnDialogEvent(long jarg1, SipCallback jarg1_, long jarg2, DialogEvent jarg2_);
  public final static native int SipCallback_OnDialogEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, DialogEvent jarg2_);
  public final static native int SipCallback_OnStackEvent(long jarg1, SipCallback jarg1_, long jarg2, StackEvent jarg2_);
  public final static native int SipCallback_OnStackEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, StackEvent jarg2_);
  public final static native int SipCallback_OnInviteEvent(long jarg1, SipCallback jarg1_, long jarg2, InviteEvent jarg2_);
  public final static native int SipCallback_OnInviteEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, InviteEvent jarg2_);
  public final static native int SipCallback_OnMessagingEvent(long jarg1, SipCallback jarg1_, long jarg2, MessagingEvent jarg2_);
  public final static native int SipCallback_OnMessagingEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, MessagingEvent jarg2_);
  public final static native int SipCallback_OnMessagingAffiliationEvent(long jarg1, SipCallback jarg1_, long jarg2, MessagingAffiliationEvent jarg2_);
  public final static native int SipCallback_OnMessagingAffiliationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, MessagingAffiliationEvent jarg2_);
  public final static native int SipCallback_OnMessagingMbmsEvent(long jarg1, SipCallback jarg1_, long jarg2, MessagingMbmsEvent jarg2_);
  public final static native int SipCallback_OnMessagingMbmsEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, MessagingMbmsEvent jarg2_);
  public final static native int SipCallback_OnMessagingLocationEvent(long jarg1, SipCallback jarg1_, long jarg2, MessagingLocationEvent jarg2_);
  public final static native int SipCallback_OnMessagingLocationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, MessagingLocationEvent jarg2_);
  public final static native int SipCallback_OnInfoEvent(long jarg1, SipCallback jarg1_, long jarg2, InfoEvent jarg2_);
  public final static native int SipCallback_OnInfoEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, InfoEvent jarg2_);
  public final static native int SipCallback_OnOptionsEvent(long jarg1, SipCallback jarg1_, long jarg2, OptionsEvent jarg2_);
  public final static native int SipCallback_OnOptionsEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, OptionsEvent jarg2_);
  public final static native int SipCallback_OnPublicationEvent(long jarg1, SipCallback jarg1_, long jarg2, PublicationEvent jarg2_);
  public final static native int SipCallback_OnPublicationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, PublicationEvent jarg2_);
  public final static native int SipCallback_OnPublicationAffiliationEvent(long jarg1, SipCallback jarg1_, long jarg2, PublicationAffiliationEvent jarg2_);
  public final static native int SipCallback_OnPublicationAffiliationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, PublicationAffiliationEvent jarg2_);
  public final static native int SipCallback_OnPublicationAuthenticationEvent(long jarg1, SipCallback jarg1_, long jarg2, PublicationAuthenticationEvent jarg2_);
  public final static native int SipCallback_OnPublicationAuthenticationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, PublicationAuthenticationEvent jarg2_);
  public final static native int SipCallback_OnRegistrationAuthenticationEvent(long jarg1, SipCallback jarg1_, long jarg2, RegistrationAuthenticationEvent jarg2_);
  public final static native int SipCallback_OnRegistrationAuthenticationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, RegistrationAuthenticationEvent jarg2_);
  public final static native int SipCallback_OnRegistrationEvent(long jarg1, SipCallback jarg1_, long jarg2, RegistrationEvent jarg2_);
  public final static native int SipCallback_OnRegistrationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, RegistrationEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionEvent(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionAffiliationEvent(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionAffiliationEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionAffiliationEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionAffiliationEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionCMSEvent(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionCMSEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionCMSEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionCMSEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionGMSEvent(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionGMSEvent jarg2_);
  public final static native int SipCallback_OnSubscriptionGMSEventSwigExplicitSipCallback(long jarg1, SipCallback jarg1_, long jarg2, SubscriptionGMSEvent jarg2_);
  public final static native void SipCallback_director_connect(SipCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void SipCallback_change_ownership(SipCallback obj, long cptr, boolean take_or_release);
  public final static native long new_SafeObject();
  public final static native void delete_SafeObject(long jarg1);
  public final static native int SafeObject_Lock(long jarg1, SafeObject jarg1_);
  public final static native int SafeObject_UnLock(long jarg1, SafeObject jarg1_);
  public final static native long new_SipStack(long jarg1, SipCallback jarg1_, String jarg2, String jarg3, String jarg4);
  public final static native void delete_SipStack(long jarg1);
  public final static native boolean SipStack_start(long jarg1, SipStack jarg1_);
  public final static native boolean SipStack_setDebugCallback(long jarg1, SipStack jarg1_, long jarg2, DDebugCallback jarg2_);
  public final static native boolean SipStack_setRegisterCallback(long jarg1, SipStack jarg1_, long jarg2, DRegisterCallback jarg2_, java.nio.ByteBuffer jarg3, int jarg4);
  public final static native boolean SipStack_setDisplayName(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setRealm(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setIMPI(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setIMPU(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setIMPUIP(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setPassword(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setAMF(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setOperatorId(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setProxyCSCF(long jarg1, SipStack jarg1_, String jarg2, int jarg3, String jarg4, String jarg5);
  public final static native boolean SipStack_setLocalIP__SWIG_0(long jarg1, SipStack jarg1_, String jarg2, String jarg3);
  public final static native boolean SipStack_setLocalIP__SWIG_1(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setLocalPort__SWIG_0(long jarg1, SipStack jarg1_, int jarg2, String jarg3);
  public final static native boolean SipStack_setLocalPort__SWIG_1(long jarg1, SipStack jarg1_, int jarg2);
  public final static native boolean SipStack_setEarlyIMS(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_addHeader(long jarg1, SipStack jarg1_, String jarg2, String jarg3);
  public final static native boolean SipStack_removeHeader(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_addDnsServer(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setDnsDiscovery(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setAoR(long jarg1, SipStack jarg1_, String jarg2, int jarg3);
  public final static native boolean SipStack_setSigCompParams(long jarg1, SipStack jarg1_, long jarg2, long jarg3, long jarg4, boolean jarg5);
  public final static native boolean SipStack_addSigCompCompartment(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_removeSigCompCompartment(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setSTUNEnabledForICE(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setSTUNServer(long jarg1, SipStack jarg1_, String jarg2, int jarg3);
  public final static native boolean SipStack_setSTUNCred(long jarg1, SipStack jarg1_, String jarg2, String jarg3);
  public final static native boolean SipStack_setSTUNEnabled(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setTLSSecAgree(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setSSLCertificates__SWIG_0(long jarg1, SipStack jarg1_, String jarg2, String jarg3, String jarg4, boolean jarg5);
  public final static native boolean SipStack_setSSLCertificates__SWIG_1(long jarg1, SipStack jarg1_, String jarg2, String jarg3, String jarg4);
  public final static native boolean SipStack_setSSLCretificates__SWIG_0(long jarg1, SipStack jarg1_, String jarg2, String jarg3, String jarg4, boolean jarg5);
  public final static native boolean SipStack_setSSLCretificates__SWIG_1(long jarg1, SipStack jarg1_, String jarg2, String jarg3, String jarg4);
  public final static native boolean SipStack_setIPSecSecAgree(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setIPSecParameters(long jarg1, SipStack jarg1_, String jarg2, String jarg3, String jarg4, String jarg5);
  public final static native boolean SipStack_setMCPTTPSIPrivate(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTPSIGroup(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTPSIPreestablished(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTPSICMS(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTPSIGMS(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTID(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTClientID(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTPriority(long jarg1, SipStack jarg1_, int jarg2);
  public final static native boolean SipStack_setMCPTTImplicit(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTGranted(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTAnswerMode(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTPrivAnswerMode(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTNameSpace(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTInsertXFramedIP(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTTimerT100(long jarg1, SipStack jarg1_, int jarg2);
  public final static native boolean SipStack_setMCPTTTimerT101(long jarg1, SipStack jarg1_, int jarg2);
  public final static native boolean SipStack_setMCPTTTimerT103(long jarg1, SipStack jarg1_, int jarg2);
  public final static native boolean SipStack_setMCPTTTimerT104(long jarg1, SipStack jarg1_, int jarg2);
  public final static native boolean SipStack_setMCPTTTimerT132(long jarg1, SipStack jarg1_, int jarg2);
  public final static native boolean SipStack_setLocationPAssertedIdentityServer(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMbmsPAssertedIdentityServer(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMbmsPortManager(long jarg1, SipStack jarg1_, long jarg2);
  public final static native boolean SipStack_setMbmsAddrManager(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMbmsIsRTCPMux(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTPSIAffiliation(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTAffiliationIsEnable(long jarg1, SipStack jarg1_, boolean jarg2);
  public final static native boolean SipStack_setMCPTTAffiliationGroupsDefualt(long jarg1, SipStack jarg1_, String jarg2);
  public final static native boolean SipStack_setMCPTTPSIAuthentication(long jarg1, SipStack jarg1_, String jarg2);
  public final static native String SipStack_dnsENUM(long jarg1, SipStack jarg1_, String jarg2, String jarg3, String jarg4);
  public final static native String SipStack_dnsNaptrSrv(long jarg1, SipStack jarg1_, String jarg2, String jarg3, int[] jarg4);
  public final static native String SipStack_dnsSrv(long jarg1, SipStack jarg1_, String jarg2, int[] jarg3);
  public final static native boolean SipStack_setMaxFDs(long jarg1, SipStack jarg1_, long jarg2);
  public final static native String SipStack_getLocalIPnPort(long jarg1, SipStack jarg1_, String jarg2, int[] jarg3);
  public final static native String SipStack_getPreferredIdentity(long jarg1, SipStack jarg1_);
  public final static native boolean SipStack_isValid(long jarg1, SipStack jarg1_);
  public final static native boolean SipStack_stop(long jarg1, SipStack jarg1_);
  public final static native boolean SipStack_initialize();
  public final static native boolean SipStack_deInitialize();
  public final static native void SipStack_setCodecs(int jarg1);
  public final static native void SipStack_setCodecs_2(long jarg1);
  public final static native boolean SipStack_setCodecPriority(int jarg1, int jarg2);
  public final static native boolean SipStack_setCodecPriority_2(int jarg1, int jarg2);
  public final static native boolean SipStack_isCodecSupported(int jarg1);
  public final static native boolean SipStack_isIPSecSupported();
  public final static native long new_XcapSelector(long jarg1, XcapStack jarg1_);
  public final static native void delete_XcapSelector(long jarg1);
  public final static native long XcapSelector_setAUID(long jarg1, XcapSelector jarg1_, String jarg2);
  public final static native long XcapSelector_setName(long jarg1, XcapSelector jarg1_, String jarg2);
  public final static native long XcapSelector_setAttribute(long jarg1, XcapSelector jarg1_, String jarg2, String jarg3, String jarg4);
  public final static native long XcapSelector_setPos(long jarg1, XcapSelector jarg1_, String jarg2, long jarg3);
  public final static native long XcapSelector_setPosAttribute(long jarg1, XcapSelector jarg1_, String jarg2, long jarg3, String jarg4, String jarg5);
  public final static native long XcapSelector_setNamespace(long jarg1, XcapSelector jarg1_, String jarg2, String jarg3);
  public final static native String XcapSelector_getString(long jarg1, XcapSelector jarg1_);
  public final static native void XcapSelector_reset(long jarg1, XcapSelector jarg1_);
  public final static native long new_XcapMessage();
  public final static native void delete_XcapMessage(long jarg1);
  public final static native short XcapMessage_getCode(long jarg1, XcapMessage jarg1_);
  public final static native String XcapMessage_getPhrase(long jarg1, XcapMessage jarg1_);
  public final static native String XcapMessage_getXcapHeaderValue__SWIG_0(long jarg1, XcapMessage jarg1_, String jarg2, long jarg3);
  public final static native String XcapMessage_getXcapHeaderValue__SWIG_1(long jarg1, XcapMessage jarg1_, String jarg2);
  public final static native String XcapMessage_getXcapHeaderParamValue__SWIG_0(long jarg1, XcapMessage jarg1_, String jarg2, String jarg3, long jarg4);
  public final static native String XcapMessage_getXcapHeaderParamValue__SWIG_1(long jarg1, XcapMessage jarg1_, String jarg2, String jarg3);
  public final static native long XcapMessage_getXcapContentLength(long jarg1, XcapMessage jarg1_);
  public final static native long XcapMessage_getXcapContent(long jarg1, XcapMessage jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native void delete_XcapEvent(long jarg1);
  public final static native int XcapEvent_getType(long jarg1, XcapEvent jarg1_);
  public final static native long XcapEvent_getXcapMessage(long jarg1, XcapEvent jarg1_);
  public final static native long new_XcapCallback();
  public final static native void delete_XcapCallback(long jarg1);
  public final static native int XcapCallback_onEvent(long jarg1, XcapCallback jarg1_, long jarg2, XcapEvent jarg2_);
  public final static native int XcapCallback_onEventSwigExplicitXcapCallback(long jarg1, XcapCallback jarg1_, long jarg2, XcapEvent jarg2_);
  public final static native void XcapCallback_director_connect(XcapCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void XcapCallback_change_ownership(XcapCallback obj, long cptr, boolean take_or_release);
  public final static native long new_XcapStack(long jarg1, XcapCallback jarg1_, String jarg2, String jarg3, String jarg4);
  public final static native void delete_XcapStack(long jarg1);
  public final static native boolean XcapStack_registerAUID(long jarg1, XcapStack jarg1_, String jarg2, String jarg3, String jarg4, String jarg5, boolean jarg6);
  public final static native boolean XcapStack_start(long jarg1, XcapStack jarg1_);
  public final static native boolean XcapStack_setCredentials(long jarg1, XcapStack jarg1_, String jarg2, String jarg3);
  public final static native boolean XcapStack_setXcapRoot(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_setLocalIP(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_setLocalPort(long jarg1, XcapStack jarg1_, long jarg2);
  public final static native boolean XcapStack_addHeader(long jarg1, XcapStack jarg1_, String jarg2, String jarg3);
  public final static native boolean XcapStack_removeHeader(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_setTimeout(long jarg1, XcapStack jarg1_, long jarg2);
  public final static native boolean XcapStack_getDocument(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_getElement(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_getAttribute(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_deleteDocument(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_deleteElement(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_deleteAttribute(long jarg1, XcapStack jarg1_, String jarg2);
  public final static native boolean XcapStack_putDocument(long jarg1, XcapStack jarg1_, String jarg2, java.nio.ByteBuffer jarg3, long jarg4, String jarg5);
  public final static native boolean XcapStack_putElement(long jarg1, XcapStack jarg1_, String jarg2, java.nio.ByteBuffer jarg3, long jarg4);
  public final static native boolean XcapStack_putAttribute(long jarg1, XcapStack jarg1_, String jarg2, java.nio.ByteBuffer jarg3, long jarg4);
  public final static native boolean XcapStack_stop(long jarg1, XcapStack jarg1_);
  public final static native long new_RPMessage();
  public final static native void delete_RPMessage(long jarg1);
  public final static native int RPMessage_getType(long jarg1, RPMessage jarg1_);
  public final static native long RPMessage_getPayloadLength(long jarg1, RPMessage jarg1_);
  public final static native long RPMessage_getPayload(long jarg1, RPMessage jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native long new_SMSData();
  public final static native void delete_SMSData(long jarg1);
  public final static native int SMSData_getType(long jarg1, SMSData jarg1_);
  public final static native int SMSData_getMR(long jarg1, SMSData jarg1_);
  public final static native long SMSData_getPayloadLength(long jarg1, SMSData jarg1_);
  public final static native long SMSData_getPayload(long jarg1, SMSData jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native String SMSData_getOA(long jarg1, SMSData jarg1_);
  public final static native String SMSData_getDA(long jarg1, SMSData jarg1_);
  public final static native long SMSEncoder_encodeSubmit(int jarg1, String jarg2, String jarg3, String jarg4);
  public final static native long SMSEncoder_encodeDeliver(int jarg1, String jarg2, String jarg3, String jarg4);
  public final static native long SMSEncoder_encodeACK(int jarg1, String jarg2, String jarg3, boolean jarg4);
  public final static native long SMSEncoder_encodeError(int jarg1, String jarg2, String jarg3, boolean jarg4);
  public final static native long SMSEncoder_decode(java.nio.ByteBuffer jarg1, long jarg2, boolean jarg3);
  public final static native void delete_SMSEncoder(long jarg1);
  public final static native long new_MsrpMessage();
  public final static native void delete_MsrpMessage(long jarg1);
  public final static native boolean MsrpMessage_isRequest(long jarg1, MsrpMessage jarg1_);
  public final static native short MsrpMessage_getCode(long jarg1, MsrpMessage jarg1_);
  public final static native String MsrpMessage_getPhrase(long jarg1, MsrpMessage jarg1_);
  public final static native int MsrpMessage_getRequestType(long jarg1, MsrpMessage jarg1_);
  public final static native void MsrpMessage_getByteRange(long jarg1, MsrpMessage jarg1_, long[] jarg2, long[] jarg3, long[] jarg4);
  public final static native boolean MsrpMessage_isLastChunck(long jarg1, MsrpMessage jarg1_);
  public final static native boolean MsrpMessage_isFirstChunck(long jarg1, MsrpMessage jarg1_);
  public final static native boolean MsrpMessage_isSuccessReport(long jarg1, MsrpMessage jarg1_);
  public final static native String MsrpMessage_getMsrpHeaderValue(long jarg1, MsrpMessage jarg1_, String jarg2);
  public final static native String MsrpMessage_getMsrpHeaderParamValue(long jarg1, MsrpMessage jarg1_, String jarg2, String jarg3);
  public final static native long MsrpMessage_getMsrpContentLength(long jarg1, MsrpMessage jarg1_);
  public final static native long MsrpMessage_getMsrpContent(long jarg1, MsrpMessage jarg1_, java.nio.ByteBuffer jarg2, long jarg3);
  public final static native void delete_MsrpEvent(long jarg1);
  public final static native int MsrpEvent_getType(long jarg1, MsrpEvent jarg1_);
  public final static native long MsrpEvent_getSipSession(long jarg1, MsrpEvent jarg1_);
  public final static native long MsrpEvent_getMessage(long jarg1, MsrpEvent jarg1_);
  public final static native long new_MsrpCallback();
  public final static native void delete_MsrpCallback(long jarg1);
  public final static native int MsrpCallback_OnEvent(long jarg1, MsrpCallback jarg1_, long jarg2, MsrpEvent jarg2_);
  public final static native int MsrpCallback_OnEventSwigExplicitMsrpCallback(long jarg1, MsrpCallback jarg1_, long jarg2, MsrpEvent jarg2_);
  public final static native void MsrpCallback_director_connect(MsrpCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void MsrpCallback_change_ownership(MsrpCallback obj, long cptr, boolean take_or_release);
  public final static native long new_McpttMessage();
  public final static native void delete_McpttMessage(long jarg1);
  public final static native short McpttMessage_getRCode(long jarg1, McpttMessage jarg1_);
  public final static native String McpttMessage_getPhrase(long jarg1, McpttMessage jarg1_);
  public final static native String McpttMessage_getUser(long jarg1, McpttMessage jarg1_);
  public final static native short McpttMessage_getParticipants(long jarg1, McpttMessage jarg1_);
  public final static native short McpttMessage_getTime(long jarg1, McpttMessage jarg1_);
  public final static native void delete_McpttEvent(long jarg1);
  public final static native int McpttEvent_getType(long jarg1, McpttEvent jarg1_);
  public final static native long McpttEvent_getSipSession(long jarg1, McpttEvent jarg1_);
  public final static native long McpttEvent_getMessage(long jarg1, McpttEvent jarg1_);
  public final static native long new_McpttCallback();
  public final static native void delete_McpttCallback(long jarg1);
  public final static native int McpttCallback_OnEvent(long jarg1, McpttCallback jarg1_, long jarg2, McpttEvent jarg2_);
  public final static native int McpttCallback_OnEventSwigExplicitMcpttCallback(long jarg1, McpttCallback jarg1_, long jarg2, McpttEvent jarg2_);
  public final static native void McpttCallback_director_connect(McpttCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void McpttCallback_change_ownership(McpttCallback obj, long cptr, boolean take_or_release);
  public final static native long new_McpttMbmsMessage();
  public final static native void delete_McpttMbmsMessage(long jarg1);
  public final static native String McpttMbmsMessage_getGroupId(long jarg1, McpttMbmsMessage jarg1_);
  public final static native String McpttMbmsMessage_getTMGI(long jarg1, McpttMbmsMessage jarg1_);
  public final static native String McpttMbmsMessage_getMediaIP(long jarg1, McpttMbmsMessage jarg1_);
  public final static native short McpttMbmsMessage_getMediaPort(long jarg1, McpttMbmsMessage jarg1_);
  public final static native short McpttMbmsMessage_getMediaControlPort(long jarg1, McpttMbmsMessage jarg1_);
  public final static native void delete_McpttMbmsEvent(long jarg1);
  public final static native int McpttMbmsEvent_getType(long jarg1, McpttMbmsEvent jarg1_);
  public final static native long McpttMbmsEvent_getSipSession(long jarg1, McpttMbmsEvent jarg1_);
  public final static native long McpttMbmsEvent_getMessage(long jarg1, McpttMbmsEvent jarg1_);
  public final static native long new_McpttMbmsCallback();
  public final static native void delete_McpttMbmsCallback(long jarg1);
  public final static native int McpttMbmsCallback_OnEvent(long jarg1, McpttMbmsCallback jarg1_, long jarg2, McpttMbmsEvent jarg2_);
  public final static native int McpttMbmsCallback_OnEventSwigExplicitMcpttMbmsCallback(long jarg1, McpttMbmsCallback jarg1_, long jarg2, McpttMbmsEvent jarg2_);
  public final static native void McpttMbmsCallback_director_connect(McpttMbmsCallback obj, long cptr, boolean mem_own, boolean weak_global);
  public final static native void McpttMbmsCallback_change_ownership(McpttMbmsCallback obj, long cptr, boolean take_or_release);
  public final static native long MediaContentCPIM_SWIGUpcast(long jarg1);
  public final static native long DialogEvent_SWIGUpcast(long jarg1);
  public final static native long StackEvent_SWIGUpcast(long jarg1);
  public final static native long InviteEvent_SWIGUpcast(long jarg1);
  public final static native long MessagingEvent_SWIGUpcast(long jarg1);
  public final static native long MessagingLocationEvent_SWIGUpcast(long jarg1);
  public final static native long MessagingAffiliationEvent_SWIGUpcast(long jarg1);
  public final static native long MessagingMbmsEvent_SWIGUpcast(long jarg1);
  public final static native long InfoEvent_SWIGUpcast(long jarg1);
  public final static native long OptionsEvent_SWIGUpcast(long jarg1);
  public final static native long PublicationEvent_SWIGUpcast(long jarg1);
  public final static native long PublicationAffiliationEvent_SWIGUpcast(long jarg1);
  public final static native long PublicationAuthenticationEvent_SWIGUpcast(long jarg1);
  public final static native long RegistrationAuthenticationEvent_SWIGUpcast(long jarg1);
  public final static native long RegistrationEvent_SWIGUpcast(long jarg1);
  public final static native long SubscriptionEvent_SWIGUpcast(long jarg1);
  public final static native long SubscriptionAffiliationEvent_SWIGUpcast(long jarg1);
  public final static native long SubscriptionCMSEvent_SWIGUpcast(long jarg1);
  public final static native long SubscriptionGMSEvent_SWIGUpcast(long jarg1);
  public final static native long InviteSession_SWIGUpcast(long jarg1);
  public final static native long CallSession_SWIGUpcast(long jarg1);
  public final static native long MsrpSession_SWIGUpcast(long jarg1);
  public final static native long McpttSession_SWIGUpcast(long jarg1);
  public final static native long MessagingSession_SWIGUpcast(long jarg1);
  public final static native long MessagingLocationSession_SWIGUpcast(long jarg1);
  public final static native long MessagingAffiliationSession_SWIGUpcast(long jarg1);
  public final static native long MessagingMbmsSession_SWIGUpcast(long jarg1);
  public final static native long InfoSession_SWIGUpcast(long jarg1);
  public final static native long OptionsSession_SWIGUpcast(long jarg1);
  public final static native long PublicationSession_SWIGUpcast(long jarg1);
  public final static native long PublicationAffiliationSession_SWIGUpcast(long jarg1);
  public final static native long PublicationAuthenticationSession_SWIGUpcast(long jarg1);
  public final static native long RegistrationSession_SWIGUpcast(long jarg1);
  public final static native long SubscriptionSession_SWIGUpcast(long jarg1);
  public final static native long SubscriptionAffiliationSession_SWIGUpcast(long jarg1);
  public final static native long SubscriptionCMSSession_SWIGUpcast(long jarg1);
  public final static native long SubscriptionGMSSession_SWIGUpcast(long jarg1);
  public final static native long ProxyAudioConsumer_SWIGUpcast(long jarg1);
  public final static native long ProxyVideoConsumer_SWIGUpcast(long jarg1);
  public final static native long ProxyAudioProducer_SWIGUpcast(long jarg1);
  public final static native long ProxyVideoProducer_SWIGUpcast(long jarg1);
  public final static native long SipStack_SWIGUpcast(long jarg1);

  public static int SwigDirector_DDebugCallback_OnDebugTest(DDebugCallback self, String message) {
    return self.OnDebugTest(message);
  }
  public static int SwigDirector_DDebugCallback_OnDebugInfo(DDebugCallback self, String message) {
    return self.OnDebugInfo(message);
  }
  public static int SwigDirector_DDebugCallback_OnDebugWarn(DDebugCallback self, String message) {
    return self.OnDebugWarn(message);
  }
  public static int SwigDirector_DDebugCallback_OnDebugError(DDebugCallback self, String message) {
    return self.OnDebugError(message);
  }
  public static int SwigDirector_DDebugCallback_OnDebugFatal(DDebugCallback self, String message) {
    return self.OnDebugFatal(message);
  }
  public static int SwigDirector_DRegisterCallback_onAuthRegister(DRegisterCallback self, String message) {
    return self.onAuthRegister(message);
  }
  public static int SwigDirector_T140Callback_ondata(T140Callback self, long pData) {
    return self.ondata((pData == 0) ? null : new T140CallbackData(pData, false));
  }
  public static int SwigDirector_ProxyPluginMgrCallback_OnPluginCreated(ProxyPluginMgrCallback self, java.math.BigInteger id, int type) {
    return self.OnPluginCreated(id, twrap_proxy_plugin_type_t.swigToEnum(type));
  }
  public static int SwigDirector_ProxyPluginMgrCallback_OnPluginDestroyed(ProxyPluginMgrCallback self, java.math.BigInteger id, int type) {
    return self.OnPluginDestroyed(id, twrap_proxy_plugin_type_t.swigToEnum(type));
  }
  public static int SwigDirector_ProxyAudioConsumerCallback_prepare(ProxyAudioConsumerCallback self, int ptime, int rate, int channels) {
    return self.prepare(ptime, rate, channels);
  }
  public static int SwigDirector_ProxyAudioConsumerCallback_start(ProxyAudioConsumerCallback self) {
    return self.start();
  }
  public static int SwigDirector_ProxyAudioConsumerCallback_pause(ProxyAudioConsumerCallback self) {
    return self.pause();
  }
  public static int SwigDirector_ProxyAudioConsumerCallback_stop(ProxyAudioConsumerCallback self) {
    return self.stop();
  }
  public static int SwigDirector_ProxyVideoConsumerCallback_prepare(ProxyVideoConsumerCallback self, int nWidth, int nHeight, int nFps) {
    return self.prepare(nWidth, nHeight, nFps);
  }
  public static int SwigDirector_ProxyVideoConsumerCallback_consume(ProxyVideoConsumerCallback self, long frame) {
    return self.consume((frame == 0) ? null : new ProxyVideoFrame(frame, false));
  }
  public static int SwigDirector_ProxyVideoConsumerCallback_bufferCopied(ProxyVideoConsumerCallback self, long nCopiedSize, long nAvailableSize) {
    return self.bufferCopied(nCopiedSize, nAvailableSize);
  }
  public static int SwigDirector_ProxyVideoConsumerCallback_start(ProxyVideoConsumerCallback self) {
    return self.start();
  }
  public static int SwigDirector_ProxyVideoConsumerCallback_pause(ProxyVideoConsumerCallback self) {
    return self.pause();
  }
  public static int SwigDirector_ProxyVideoConsumerCallback_stop(ProxyVideoConsumerCallback self) {
    return self.stop();
  }
  public static int SwigDirector_ProxyAudioProducerCallback_prepare(ProxyAudioProducerCallback self, int ptime, int rate, int channels) {
    return self.prepare(ptime, rate, channels);
  }
  public static int SwigDirector_ProxyAudioProducerCallback_start(ProxyAudioProducerCallback self) {
    return self.start();
  }
  public static int SwigDirector_ProxyAudioProducerCallback_pause(ProxyAudioProducerCallback self) {
    return self.pause();
  }
  public static int SwigDirector_ProxyAudioProducerCallback_stop(ProxyAudioProducerCallback self) {
    return self.stop();
  }
  public static int SwigDirector_ProxyAudioProducerCallback_fillPushBuffer(ProxyAudioProducerCallback self) {
    return self.fillPushBuffer();
  }
  public static int SwigDirector_ProxyVideoProducerCallback_prepare(ProxyVideoProducerCallback self, int width, int height, int fps) {
    return self.prepare(width, height, fps);
  }
  public static int SwigDirector_ProxyVideoProducerCallback_start(ProxyVideoProducerCallback self) {
    return self.start();
  }
  public static int SwigDirector_ProxyVideoProducerCallback_pause(ProxyVideoProducerCallback self) {
    return self.pause();
  }
  public static int SwigDirector_ProxyVideoProducerCallback_stop(ProxyVideoProducerCallback self) {
    return self.stop();
  }
  public static int SwigDirector_SipCallback_OnDialogEvent(SipCallback self, long e) {
    return self.OnDialogEvent((e == 0) ? null : new DialogEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnStackEvent(SipCallback self, long e) {
    return self.OnStackEvent((e == 0) ? null : new StackEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnInviteEvent(SipCallback self, long e) {
    return self.OnInviteEvent((e == 0) ? null : new InviteEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnMessagingEvent(SipCallback self, long e) {
    return self.OnMessagingEvent((e == 0) ? null : new MessagingEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnMessagingAffiliationEvent(SipCallback self, long e) {
    return self.OnMessagingAffiliationEvent((e == 0) ? null : new MessagingAffiliationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnMessagingMbmsEvent(SipCallback self, long e) {
    return self.OnMessagingMbmsEvent((e == 0) ? null : new MessagingMbmsEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnMessagingLocationEvent(SipCallback self, long e) {
    return self.OnMessagingLocationEvent((e == 0) ? null : new MessagingLocationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnInfoEvent(SipCallback self, long e) {
    return self.OnInfoEvent((e == 0) ? null : new InfoEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnOptionsEvent(SipCallback self, long e) {
    return self.OnOptionsEvent((e == 0) ? null : new OptionsEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnPublicationEvent(SipCallback self, long e) {
    return self.OnPublicationEvent((e == 0) ? null : new PublicationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnPublicationAffiliationEvent(SipCallback self, long e) {
    return self.OnPublicationAffiliationEvent((e == 0) ? null : new PublicationAffiliationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnPublicationAuthenticationEvent(SipCallback self, long e) {
    return self.OnPublicationAuthenticationEvent((e == 0) ? null : new PublicationAuthenticationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnRegistrationAuthenticationEvent(SipCallback self, long e) {
    return self.OnRegistrationAuthenticationEvent((e == 0) ? null : new RegistrationAuthenticationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnRegistrationEvent(SipCallback self, long e) {
    return self.OnRegistrationEvent((e == 0) ? null : new RegistrationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnSubscriptionEvent(SipCallback self, long e) {
    return self.OnSubscriptionEvent((e == 0) ? null : new SubscriptionEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnSubscriptionAffiliationEvent(SipCallback self, long e) {
    return self.OnSubscriptionAffiliationEvent((e == 0) ? null : new SubscriptionAffiliationEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnSubscriptionCMSEvent(SipCallback self, long e) {
    return self.OnSubscriptionCMSEvent((e == 0) ? null : new SubscriptionCMSEvent(e, false));
  }
  public static int SwigDirector_SipCallback_OnSubscriptionGMSEvent(SipCallback self, long e) {
    return self.OnSubscriptionGMSEvent((e == 0) ? null : new SubscriptionGMSEvent(e, false));
  }
  public static int SwigDirector_XcapCallback_onEvent(XcapCallback self, long e) {
    return self.onEvent((e == 0) ? null : new XcapEvent(e, false));
  }
  public static int SwigDirector_MsrpCallback_OnEvent(MsrpCallback self, long e) {
    return self.OnEvent((e == 0) ? null : new MsrpEvent(e, false));
  }
  public static int SwigDirector_McpttCallback_OnEvent(McpttCallback self, long e) {
    return self.OnEvent((e == 0) ? null : new McpttEvent(e, false));
  }
  public static int SwigDirector_McpttMbmsCallback_OnEvent(McpttMbmsCallback self, long e) {
    return self.OnEvent((e == 0) ? null : new McpttMbmsEvent(e, false));
  }

  private final static native void swig_module_init();
  static {
    swig_module_init();
  }
}
