/*
* Copyright (C) 2020, University of the Basque Country (UPV/EHU)
*  Contact for licensing options: <licensing-mcpttclient(at)mcopenplatform(dot)com>
*
* The original file was part of Open Source IMSDROID
*  Copyright (C) 2010-2011, Mamadou Diop.
*  Copyright (C) 2011, Doubango Telecom.
*
*
* Contact: Mamadou Diop <diopmamadou(at)doubango(dot)org>
*
* This file is part of Open Source Doubango Framework.
*
* This is free software: you can redistribute it and/or modify it under the terms of
* the GNU General Public License as published by the Free Software Foundation, either version 3
* of the License, or (at your option) any later version.
*
* This is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
* without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program; if not, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package org.doubango.ngn.utils;

public class NgnContentType {
	public final static String TEXT_PLAIN = "text/plain";
	public final static String CONFERENCE_INFO = "application/conference-info+xml";
	public final static String DIALOG_INFO = "application/dialog-info+xml";
	public final static String EXTERNAL_BODY = "message/external-body";
	public final static String MESSAGE_SUMMARY = "application/simple-message-summary";
	public final static String MULTIPART_RELATED = "multipart/related";
	public final static String OMA_DEFERRED_LIST = "application/vnd.oma.im.deferred-list+xml";
	public final static String PIDF = "application/pidf+xml";
	public final static String REG_INFO = "application/reginfo+xml";
	public final static String RLMI = "application/rlmi+xml";
	public final static String RPID = "application/rpid+xml";
	public final static String CPIM = "message/CPIM";
	
	public final static String WATCHER_INFO = "application/watcherinfo+xml";
	public final static String XCAP_DIFF = "application/xcap-diff+xml";	
	
	public final static String SMS_3GPP = "application/vnd.3gpp.sms";
	
	public final static String JSON = "application/json";
	
	public final static String DOUBANGO_DEVICE_INFO = "doubango/device-info";
	
	public final static String T140COMMAND = "doubango/t140-command";
}
